#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/select.h>
#include <string.h>

#include "icc.h"
#include "hal_interrupt_cause.h"
#include "docint.h"
#include "mlx.h"
#include "hal_global.h"
#include "init_process.h"
#include "docsis_logger.h"
#include "logger.h"
#include "docsis_shared_dbs.h"
#include "pcdapi.h"

/* List of destination points that receive ICC messages */
extern DST_t mlxtbl[];

void
multiplex(int nfd, int *fds, int maxfd)
{
	fd_set rfds;
	int i;
	short cause;
	char mask;
	ii_t ii;
	ICC_message_t *mp;
	ICC_context_t from = 777;

	/*
	 * Assuming no priority, or higher priority to file
	 * descriptors which have lower index in the list
	 */
    while (1) {
        FD_ZERO(&rfds);
        for (i = 0; i < nfd; i++)
            FD_SET(fds[i], &rfds);

        if (select(maxfd + 1, &rfds, NULL, NULL, NULL) == -1)
            break;
		for (i = 0; i < nfd; i++) {
/*			printf("Testing fd %d\n", i);*/
			if (!FD_ISSET(fds[i], &rfds))
				continue;
			if (read(fds[i], &ii, sizeof(ii)) != sizeof(ii)) {
				fprintf(stderr, "Error %d in read on fd %d\n", errno, fds[i]);
				continue;
			}
			cause = ii.ii_inum;
			mask = ii.ii_masked;
/*			printf("Call for userland handler for line %d, cause %d, masked %d\n",
				i, cause, mask); */
			if (cause < 0 || cause >= MAX_CAUSE) {
				printf("multiplex: Got bad cause %d - ignoring it\n", cause);
				continue;
			}
			if (mlxtbl[cause].d_func)
				mlxtbl[cause].d_func(NULL);
			else if(mlxtbl[cause].d_destp != MLX_NO_DEST_CONTEXT) 
            {
				if ((mp = ICC_alloc_message(mlxtbl[cause].d_destp, sizeof(ii_t), 1)) == NULL) {
					printf("multuplex: failed to alloc message\n");
					continue;
				}
				memcpy(mp->data, &ii, sizeof(ii_t));
				mp->from =  from;
				if (ICC_send_message(mlxtbl[cause].d_destp, mp, 1) == -1) {
					printf("multuplex: failed send message\n");
                    ICC_dispose_message( mp );
					continue;
				}
			}
            else
            {
               TI_LOG_WARNING(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0,
                   "*******************************************************************");
               TI_LOG_WARNING(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0,
                   "UNHANDLED INTERRUPT: No callback or destination point for cause = %d", cause);
               TI_LOG_WARNING(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0,
                   "*******************************************************************");
                continue;
            }
		}
    }

	ICC_delete_destination_point(from);
	for (i = 0; i < MAX_CAUSE; i++)
		ICC_delete_destination_point(mlxtbl[i].d_destp);

	fprintf(stderr, "select failed due to %d\n", errno);
	exit(-6);
}

/*inittable()
{
	int i;

	if (ICC_init() == -1) {
		printf("inittable: failed to init %d\n", errno);
		exit(-4);
	}

	for (i = 0; i < MAX_CAUSE; i++)
		ICC_create_destination_point(&mlxtbl[i].d_destp);
}
*/

int usage(char *name)
{
	fprintf(stderr, "%s usage: -d dev-name -n number of devices \n", name);
	exit(-1);
}

int main(int argc, char **argv)
{
	int i, maxfd = -1, nfd = 0, *fds = NULL;
	char *dev_name = NULL;

    if( PCD_api_find_process_id(argv[0]) )
    {
        fprintf( stderr, "Multiplexer process already active");
        return 1;
    }

    PCD_api_register_exception_handlers( argv[0], NULL );

	while ((i = getopt(argc, argv, "d:n:")) != -1)
		switch (i) {
		case 'd':
			dev_name = optarg;
			break;

		case 'n':
			if ((nfd = atoi(optarg)) <= 0)
				usage(argv[0]);
			break;

		default:
			usage(argv[0]);
		}

	if (dev_name == NULL || nfd <= 0)
		usage(argv[0]);

	if ((fds = (int *)calloc(nfd, sizeof(int))) == NULL) {
		fprintf(stderr, "Can't alloc for fds\n");
		exit(-2);
	}

	for (i = 0; i < nfd; i++) {
		char dname[128];

		sprintf(dname, "/dev/%s%d", dev_name, i);
		if ((fds[i] = open(dname, O_RDONLY)) < 0) {
			fprintf(stderr, "Can't open %s\n", dname);
			exit(-3);
		}
        if (fds[i] > maxfd)
			maxfd = fds[i];
	}

	if (ICC_init() == -1) {
		printf("failed to init ICC, errno %d\n", errno);
		exit(-4);
	}

    INIT_PROCESS(INIT_ALL_ITEMS);

    if (DDB_RetrieveAccess() != STATUS_OK)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_DMD, 0,
                     "MLX Failed to Retrieve Access to DOCSIS DBs.");
        return -1;
    }

    if( HAL_GlobalRegistration(True) != STATUS_OK )
    {
        printf("Global registration failed\n");
        exit(-5);
    }

    PCD_api_send_process_ready();

    multiplex(nfd, fds, maxfd);

    return 0;
}
