#include <stdio.h>

#include "sys_ptypes.h"
#include "events.h"
#include "icc.h"
#include "logger.h"
#include "status.h"


#include "bpi_interface.h"
#include "channel_manager.h"
#include "cmd_mbox.h"
#include "context_owners.h"
#include "docint.h"
#include "docsis_logger.h"
#include "downstream_manager_api.h"
#include "event_mbox.h"
#include "hal_bpi.h"
#include "hal_db.h"
#include "hal_ds_channel_control.h"
#include "hal_interrupt_cause.h"
#include "hal_mdd.h"
#include "hal_phy_regs.h"
#include "hal_reg_access.h"
#include "hal_sync.h"
#include "mdd_handler.h"
#include "mlx.h"
#include "us_handler_api.h"
#include "us_power_control_manager.h"

/*#define DEBUG_TLB_PRINTS*/

/**************************************************************************/
/*      LOCAL DECLARATIONS:                                               */
/**************************************************************************/
static STATUS HAL_IsrSyncLossHandler();
static STATUS HAL_IsrPhyErrUs4Handler();
static STATUS HAL_IsrPhyErrUs3Handler();
static STATUS HAL_IsrPhyErrUs2Handler();
static STATUS HAL_IsrPhyErrUs1Handler();
static STATUS HAL_IsrPhyErrHandler(Uint8 port);
static STATUS HAL_IsrRx1FecLostHandler();/*Phy-Rx1*/
static STATUS HAL_IsrRx1FecLockHandler();
static STATUS HAL_IsrRx2FecLostHandler();/*Phy-Rx2*/
static STATUS HAL_IsrRx2FecLockHandler();
static STATUS HAL_IsrRx3FecLostHandler();/*Phy-Rx3*/
static STATUS HAL_IsrRx3FecLockHandler();
static STATUS HAL_IsrRx4FecLostHandler();/*Phy-Rx4*/
static STATUS HAL_IsrRx4FecLockHandler();

static STATUS HAL_IsrBpiKeySeqErrHandler();
static STATUS HAL_IsrRx1MddArrivalHandler();/* Pdsp-Rx1 MDD Arrival */
static STATUS HAL_IsrRx2MddArrivalHandler();/* Pdsp-Rx2 MDD Arrival */
static STATUS HAL_IsrRx3MddArrivalHandler();/* Pdsp-Rx3 MDD Arrival */
static STATUS HAL_IsrRx4MddArrivalHandler();/* Pdsp-Rx4 MDD Arrival */
static STATUS HAL_IsrRx5MddArrivalHandler();/* Pdsp-Rx5 MDD Arrival */
static STATUS HAL_IsrRx6MddArrivalHandler();/* Pdsp-Rx6 MDD Arrival */
static STATUS HAL_IsrRx7MddArrivalHandler();/* Pdsp-Rx7 MDD Arrival */
static STATUS HAL_IsrRx8MddArrivalHandler();/* Pdsp-Rx8 MDD Arrival */
static STATUS HAL_IsrMddArrivalHandler(Uint8 portId);/* Pdsp-General MDD Arrival */

static STATUS HAL_IsrPgaGrtHandler();

static STATUS HAL_IsrMapLossHandlerUs1();  /* Map has not arrived for over 1 second for upstream 1 */
static STATUS HAL_IsrMapLossHandlerUs2();  /* Map has not arrived for over 1 second for upstream 2 */
static STATUS HAL_IsrMapLossHandlerUs3();  /* Map has not arrived for over 1 second for upstream 3 */
static STATUS HAL_IsrMapLossHandlerUs4();  /* Map has not arrived for over 1 second for upstream 4 */
static STATUS HAL_IsrRngReqSent1();
static STATUS HAL_IsrRngReqSent2();
static STATUS HAL_IsrRngReqSent3();
static STATUS HAL_IsrRngReqSent4();

/* interrupt cause table */
DST_t mlxtbl[MAX_CAUSE] = {
	/* MAC_ERR1_INT_GROUP */
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	/* MAC_ERR2_INT_GROUP */
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrSyncLossHandler },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrMapLossHandlerUs4 },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrMapLossHandlerUs3 },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrMapLossHandlerUs2 },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrMapLossHandlerUs1 },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrPhyErrUs4Handler},
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrPhyErrUs3Handler},
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrPhyErrUs2Handler},
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrPhyErrUs1Handler},
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	/* MAC_ERR3_INT_GROUP */
	{ MLX_NO_DEST_CONTEXT, NULL /*Rx1MddLoss*/ },
	{ MLX_NO_DEST_CONTEXT, NULL /*Rx2MddLoss*/ },
	{ MLX_NO_DEST_CONTEXT, NULL /*Rx3MddLoss*/ },
	{ MLX_NO_DEST_CONTEXT, NULL /*Rx4MddLoss*/ },
	{ MLX_NO_DEST_CONTEXT, NULL /*Rx5MddLoss*/ },
	{ MLX_NO_DEST_CONTEXT, NULL /*Rx6MddLoss*/ },
	{ MLX_NO_DEST_CONTEXT, NULL /*Rx7MddLoss*/ },
	{ MLX_NO_DEST_CONTEXT, NULL /*Rx8MddLoss*/ },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	/* MAC_ERR4_INT_GROUP */
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	/* MAC_UCD_INT_GROUP */
	{ MLX_NO_DEST_CONTEXT, (func)USM_UcdChangeMapActivePort1 },
	{ MLX_NO_DEST_CONTEXT, (func)USM_UcdChangeMapActivePort2 },
	{ MLX_NO_DEST_CONTEXT, (func)USM_UcdChangeMapActivePort3 },
	{ MLX_NO_DEST_CONTEXT, (func)USM_UcdChangeMapActivePort4 },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	/* MAC_US_FW_INT_GROUP */
	{ MLX_NO_DEST_CONTEXT, (func)HAL_CmdMbox_SendCmdCmplt },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_EventMbox_SendEventReceived },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRngReqSent1 },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRngReqSent2 },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRngReqSent3 },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRngReqSent4 },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_CmdMbox_SendMboxReady },
	/* MAC_DS_FW_INT_GROUP */
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrBpiKeySeqErrHandler },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRx1MddArrivalHandler },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRx2MddArrivalHandler },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRx3MddArrivalHandler },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRx4MddArrivalHandler },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRx5MddArrivalHandler },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRx6MddArrivalHandler },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRx7MddArrivalHandler },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRx8MddArrivalHandler },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_MacDsSeqOutOfRangeIsrHandler },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	/* PHY_INT_GROUP */
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRx1FecLostHandler },/*Rx1: 0-5*/
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRx1FecLockHandler },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRx2FecLostHandler },/*Rx2: 6-11*/
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRx2FecLockHandler },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRx3FecLostHandler },/*Rx3: 12-17*/
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRx3FecLockHandler },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRx4FecLostHandler },/*Rx4: 18-23*/
	{ MLX_NO_DEST_CONTEXT, (func)HAL_IsrRx4FecLockHandler },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },/*24-28*/
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },/*spare*/
	{ MLX_NO_DEST_CONTEXT, NULL },
	{ MLX_NO_DEST_CONTEXT, NULL },
    /* PGA GRT INTERRUPT */
	{ MLX_NO_DEST_CONTEXT, HAL_IsrPgaGrtHandler }, /*TBD*/
};



/**************************************************************************/
/*      LOCAL FUNCTIONS:                                                  */
/**************************************************************************/
static STATUS HAL_IsrSyncLossHandler()
{
    Uint32 primaryIdx;

    HAL_SyncTimeoutNotificationEnable(False);

    /* Get the index of the primary DS */
    HalDb_GetPrimaryDsReceiver(&primaryIdx);

    TI_LOG_INFO(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, 
                "DS[%d]: SYNC Lost - Notidy to DSM.", primaryIdx+1);

    /* invoke the DS Manager's handler function */
    DSM_NotifySyncLost(primaryIdx+1);

    return STATUS_OK;
}

static STATUS HAL_IsrPhyErrUs4Handler()
{
    return HAL_IsrPhyErrHandler(3);
}

static STATUS HAL_IsrPhyErrUs3Handler()
{
    return HAL_IsrPhyErrHandler(2);
}

static STATUS HAL_IsrPhyErrUs2Handler()
{
    return HAL_IsrPhyErrHandler(1);
}

static STATUS HAL_IsrPhyErrUs1Handler()
{
    return HAL_IsrPhyErrHandler(0);
}

static STATUS HAL_IsrPhyErrHandler(Uint8 port)
{
    Uint16 reg85 = PHY_TX_CH_REG_OFFSET(port, PHY_STATUS_1_REGISTER);
    Uint16 reg86 = PHY_TX_CH_REG_OFFSET(port, PHY_STATUS_2_SCDMA_REGISTER);
    Uint16 value85 = 0;
    Uint16 value86 = 0;
    
    /* Need a dummy-write to register 0x85 to lock 0x85, 0x86 */
    /* Read From Register 0x85 clears registers 0x85, 0x86 (so first read 0x86) */
    HAL_SetPhyReg16(reg85, value85);

    HAL_GetPhyReg16(reg86, &value86);
    HAL_GetPhyReg16(reg85, &value85);

    TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "%s: port%d PHY ERROR. SCDMA Status1 register = 0x%x, SCDMA Status2 register = 0x%x", __FUNCTION__, port, value85, value86);

    if (value85 & 0x1)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_frame_in1 - Framer error. Framer gets f_start while framer in the middle of filling the frame.");
    }
    if (value85 & 0x2)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_frame_in2 - Framer error. Framer gets frame_start on a frame that transmitted.");
    }
    if (value85 & 0x4)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_frame_out - Framer error. Framer gets f_start before finish transmitting prev. frame.");
    }
    if (value85 & 0x8)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_framer_size - Framer error. Framer gets more symbols then he suppose to get.");
    }
    if (value85 & 0x10)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_frame_skip - Framer error. Framer gets f_start with frame num, which passed the frame number for transmission.");
    }
    if (value85 & 0x20)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_req_cnt - Spreader error, request count error.");
    }
    if (value85 & 0x40)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_ready - Spreader error. Ready error.");
    }
    if (value85 & 0x80)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_switch - Spreader error. Switch error.");
    }
    if (value85 & 0x100)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_switch_in_ff - Spreader error. Switch in middle of fast forward error.");
    }
    if (value85 & 0x200)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_b_param - Tx decision error. The burst parameters for the next frame are too far ahead/behind (more than 100 ms).");
    }
    if (value85 & 0x400)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_f_passed - Tx decision error. The incoming burst arrived too late - the frame has been transmitted.");
    }
    if (value85 & 0x800)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_b_start - Tx decision error. The burst parameters did not enter the framer and a new burst arrived.");
    }
    if (value85 & 0x1000)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_start_in_ucd  - Tx decision error. Error indication that during a wait for UCD change a burst start from the MAC arrived.");
    }
    if (value85 & 0x2000)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Error Rs_cnt - Rs error RS get more bytes than he supposed to get.");
    }
    if (value85 & 0x4000)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Error burst start - Burst transmission time is over - STDMA only.");
    }
    if (value85 & 0x8000)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Error state-machine - Start FIFO state machine is stuck - STDMA only");
    }
    

    if (value86 & 0x1)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Overflow EQL - Pre-equalizer overflow.");
    }
    if (value86 & 0x2)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Overflow PSF - Shaping filter overflow.");
    }
    if (value86 & 0x4)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Overflow HBF - HBF overflow.");
    }
    if (value86 & 0x8)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Overflow Upconv - Cubic overflow.");
    }
    if (value86 & 0x10)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Overflow Atten.");
    }
    if (value86 & 0x20)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Error STDMA ready.");
    }
    if (value86 & 0x40)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_a_start_fifo_s - Start FIFO - start within start before it reached BI (ATDMA).");
    }
    if (value86 & 0x80)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_a_start_fifo_of_s - Start FIFO overflow (ATDMA).");
    }
    if (value86 & 0x100)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_a_bi_miss_start_s - BI missed a start signal (ATDMA)");
    }
    if (value86 & 0x200)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_a_bi_start_s - Byte interleave received start before a block is ready (ATDMA).");
    }
    if (value86 & 0x400)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "bi_ram_empty - BI RAM empty (ATDMA).");
    }
    if (value86 & 0x800)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "amp_en - Amplifier enable (high when transmitting).");
    }
    
    TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "burst_profile_fifo = %d - Upstream current working profile.", ((value86 >> 12) & 0xF));

    return STATUS_OK;
}

/*Phy-Rx1*/
static STATUS HAL_IsrRx1FecLostHandler()
{
#ifdef DEBUG_TLB_PRINTS
    printf( "-->%s: portID = 1 \n", __FUNCTION__);
#endif

    TI_LOG_INFO(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, 
                "DS[1]: FEC Lost - Notidy to DSM.");

    DSM_NotifyQamFecLost(1);
    return STATUS_OK;
}

static STATUS HAL_IsrRx1FecLockHandler()
{
#ifdef DEBUG_TLB_PRINTS
    printf( "-->%s: portID = 1\n", __FUNCTION__);
#endif

    TI_LOG_INFO(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, 
                "DS[1]: FEC Recovered - Notidy to DSM.");

    DSM_NotifyQamFecRecovery(1);
    return STATUS_OK;
}

/*Phy-Rx2*/
static STATUS HAL_IsrRx2FecLostHandler()
{
#ifdef DEBUG_TLB_PRINTS
    printf( "-->%s: portID = 2\n", __FUNCTION__);
#endif

    TI_LOG_INFO(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, 
                "DS[2]: FEC Lost - Notidy to DSM.");

    DSM_NotifyQamFecLost(2);
    return STATUS_OK;
}

static STATUS HAL_IsrRx2FecLockHandler()
{
#ifdef DEBUG_TLB_PRINTS
    printf( "-->%s: portID = 2\n", __FUNCTION__);
#endif

    TI_LOG_INFO(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, 
                "DS[2]: FEC Recovered - Notidy to DSM.");

    DSM_NotifyQamFecRecovery(2);
    return STATUS_OK;
}

/*Phy-Rx3*/
static STATUS HAL_IsrRx3FecLostHandler()
{
#ifdef DEBUG_TLB_PRINTS
    printf( "-->%s: portID = 3\n", __FUNCTION__);
#endif

    TI_LOG_INFO(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, 
                "DS[3]: FEC Lost - Notidy to DSM.");

    DSM_NotifyQamFecLost(3);
    return STATUS_OK;
}

static STATUS HAL_IsrRx3FecLockHandler()
{
#ifdef DEBUG_TLB_PRINTS
    printf( "-->%s: portID = 3\n", __FUNCTION__);
#endif

    TI_LOG_INFO(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, 
                "DS[3]: FEC Recovered - Notidy to DSM.");

    DSM_NotifyQamFecRecovery(3);
    return STATUS_OK;
}

/*Phy-Rx4*/
static STATUS HAL_IsrRx4FecLostHandler()
{
#ifdef DEBUG_TLB_PRINTS
    printf( "-->%s\n: portID = 4\n", __FUNCTION__);
#endif

    TI_LOG_INFO(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, 
                "DS[4]: FEC Lost - Notidy to DSM.");

    DSM_NotifyQamFecLost(4);
    return STATUS_OK;
}

static STATUS HAL_IsrRx4FecLockHandler()
{
#ifdef DEBUG_TLB_PRINTS
    printf( "-->%s: portID = 4\n", __FUNCTION__);
#endif

    TI_LOG_INFO(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, 
                "DS[4]: FEC Recovered - Notidy to DSM.");

    DSM_NotifyQamFecRecovery(4);
    return STATUS_OK;
}


static STATUS HAL_IsrBpiKeySeqErrHandler()
{
    Int16 index = -1;

    /* Get the SAID from HW - TBD */
    if(HAL_BpiDsGetFirstKeySeqErrIndex(&index) != STATUS_OK)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Failed to get SAID index from HW");
        return  STATUS_NOK;
    }

    if(index == -1)
    {
        TI_LOG_WARNING(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Received Key-Seq error interrupt, but no SAID index is set in HW");
        HAL_BpiDsEnableKeySeqErrorNotification(True);
        return  STATUS_NOK;
    }
    /* Envoke the BPI API for notifying TEK-Invalid */
    if(BpiIfc_TekSeqNumInv(index) != STATUS_OK )
    {                          
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Failed to send Teq-Seq-Invalid message to BPI SM");
        return STATUS_NOK;
    }

    return STATUS_OK;
}


/* Pdsp-Rx1 MDD Arrival */
static STATUS HAL_IsrRx1MddArrivalHandler()
{
    return HAL_IsrMddArrivalHandler(1);
}

/* Pdsp-Rx2 MDD Arrival */
static STATUS HAL_IsrRx2MddArrivalHandler()
{
    return HAL_IsrMddArrivalHandler(2);
}

/* Pdsp-Rx3 MDD Arrival */
static STATUS HAL_IsrRx3MddArrivalHandler()
{
    return HAL_IsrMddArrivalHandler(3);
}

/* Pdsp-Rx4 MDD Arrival */
static STATUS HAL_IsrRx4MddArrivalHandler()
{
    return HAL_IsrMddArrivalHandler(4);
}

/* Pdsp-Rx4 MDD Arrival */
static STATUS HAL_IsrRx5MddArrivalHandler()
{
    return HAL_IsrMddArrivalHandler(5);
}

/* Pdsp-Rx4 MDD Arrival */
static STATUS HAL_IsrRx6MddArrivalHandler()
{
    return HAL_IsrMddArrivalHandler(6);
}

/* Pdsp-Rx4 MDD Arrival */
static STATUS HAL_IsrRx7MddArrivalHandler()
{
    return HAL_IsrMddArrivalHandler(7);
}

/* Pdsp-Rx4 MDD Arrival */
static STATUS HAL_IsrRx8MddArrivalHandler()
{
    return HAL_IsrMddArrivalHandler(8);
}

/* Pdsp - MDD Arrival (general) */
static STATUS HAL_IsrMddArrivalHandler(Uint8 portId)
{
    /* Note: Further notifications of the same event are masked */
    /* by the kernel-level handler. Will be unmasked again next time */
    /* an MDD-Loss event is reported by h/w. */

    /* Notify DS manager -- MDD Arrival */
    if (DSM_NotifyMddArrival(portId) != DSM_RS_OK)
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, 
                    "DS[%d]: MDD Recovered - Failed to notify DS manager!", portId);

    return STATUS_OK;
}

static STATUS HAL_IsrPgaGrtHandler()
{
    UPCM_HandleCancelWaitForGRT();
    return (STATUS_OK);
}

static STATUS HAL_IsrMapLossHandlerUs(Uint32 usPort)
{
    TI_LOG_INFO(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, 
                "US[%d]: MAPs lost", usPort);
    EV_GenEvent(ICC_dest_of(CTX_CHM), CHM_ev_MapsLost, &usPort, sizeof(usPort));

    return STATUS_OK;
}

static STATUS HAL_IsrMapLossHandlerUs1()
{
    /* call notification function here, with port=0 */
    HAL_IsrMapLossHandlerUs(1);
    return (STATUS_OK);
}

static STATUS HAL_IsrMapLossHandlerUs2()
{
    /* call notification function here, with port=1 */
    HAL_IsrMapLossHandlerUs(2);
    return (STATUS_OK);
}
static STATUS HAL_IsrMapLossHandlerUs3()
{
    /* call notification function here, with port=2 */
    HAL_IsrMapLossHandlerUs(3);
    return (STATUS_OK);
}
static STATUS HAL_IsrMapLossHandlerUs4()
{
    /* call notification function here, with port=3 */
    HAL_IsrMapLossHandlerUs(4);
    return (STATUS_OK);
}

static STATUS HAL_IsrRngReqSent1()
{
    EV_GenEvent(ICC_dest_of(CTX_USM1), USM_ev_RngReqSent, NULL, 0);

    return STATUS_OK;
}
static STATUS HAL_IsrRngReqSent2()
{
    EV_GenEvent(ICC_dest_of(CTX_USM2), USM_ev_RngReqSent, NULL, 0);

    return STATUS_OK;
}
static STATUS HAL_IsrRngReqSent3()
{
    EV_GenEvent(ICC_dest_of(CTX_USM3), USM_ev_RngReqSent, NULL, 0);

    return STATUS_OK;
}
static STATUS HAL_IsrRngReqSent4()
{
    EV_GenEvent(ICC_dest_of(CTX_USM4), USM_ev_RngReqSent, NULL, 0);

    return STATUS_OK;
}


