/*
 *
 * docsis_mng.c
 * Description:
 * DOCSIS management driver implementation
 *
 * Copyright (C) 2008 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <asm/uaccess.h>
#include <linux/kernel.h>
#include <linux/init.h> 
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/poll.h>
#include <linux/interrupt.h>
#include <linux/cdev.h>
#include <linux/sched.h>
#include <asm/io.h>
#include <linux/timer.h>
#include <linux/slab.h>
#include <linux/proc_fs.h>

#include <puma5/puma5_cppi.h>
#include "_tistdtypes.h"
#include "mng_cppi.h"
#include "hal_docsis_cppi.h"
#include "pal.h"
#include "pal_cppi41.h"
#include "pal_osWait.h"
#include "docsis_mng.h"
#include "mac_mngt_msg.h"

#define DOCSIS_MNG_MAJOR 32
#define DOCSIS_MNG_NAME "docsis_mng"

#if 0 
/*
 * old defines from old puma5_cppi.h
 */
#define DOCSIS_MNG_RX_BD_SIZE             64 /* sizeof Cppi4HostDesc + align */
#define DOCSIS_MNG_RX_BD_NUM              32 /* can we have less than that ??? */

#define DOCSIS_MNG_TX_BD_SIZE             64 /* sizeof Cppi4HostDesc + align */
#define DOCSIS_MNG_TX_BD_NUM              32 /* can we have less than that ??? */

#define DOCSIS_MNG_QMGR                   0
#define DOCSIS_MNG_FD_QNUM                137
#define DOCSIS_MNG_RX_QNUM                195
#define DOCSIS_MNG_RX_QNUM1               106

#define DOCSIS_MNG_TX_FD_QNUM             138
#define DOCSIS_MNG_TX_QNUM                180

#define DOCSIS_MNG_ACCRX_INTD_VEC         5
#define DOCSIS_MNG_ACC_RX_CHNUM           9

#else
/*
 * use new defines from puma5_cppi_prv.h
 */
#define DOCSIS_MNG_RX_BD_SIZE           CMGR_CPPI4x_RX_HOST_BD_SIZE /* sizeof Cppi4HostDesc + align */
#define DOCSIS_MNG_RX_BD_NUM            64 // cable_pp: there is no region with 32 bds, DMAC_MNG_RX_HOST_BD_NUM /* can we have less than that ??? */

#define DOCSIS_MNG_TX_BD_SIZE           CMGR_CPPI4x_TX_HOST_BD_SIZE /* sizeof Cppi4HostDesc + align */
#define DOCSIS_MNG_TX_BD_NUM            64 // cable_pp: there is no region with 32 bds, DMAC_MNG_TX_HOST_BD_NUM /* can we have less than that ??? */

#define DOCSIS_MNG_QMGR                 CMNG_CPPI4x_TX_COMP_QMGR
#define DOCSIS_MNG_FD_QNUM              137 //CMGR_CPPI4x_FBD_QNUM // !!!!!!! was 137, now 140
#define DOCSIS_MNG_RX_QNUM              CMGR_CPPI4x_RX_POOLING_QNUM
#define DOCSIS_MNG_RX_QNUM1             106 //CMGR_CPPI4x_FBD_QNUM // !!!!!!! was 106, now 109

#define DOCSIS_MNG_TX_FD_QNUM           138 // !!!!!!!! ask ziv
#define DOCSIS_MNG_TX_QNUM              CMGR_CPPI4x_TX_QNUM

#define DOCSIS_MNG_ACCRX_INTD_VEC       5 //CMNG_ACC_RX_INTV
#define DOCSIS_MNG_ACC_RX_CHNUM         9 //CMNG_ACC_RX_CHNUM

#endif




#define DOCSIS_MNG_RX_BUFFER_SIZE       2*1024 /* max msg size ???*/
#define DOCSIS_MNG_RX_BUFFER_NUM        DOCSIS_MNG_RX_BD_NUM

#define DOCSIS_MNG_TX_BUFFER_SIZE       2*1024 /* max msg size ???*/
#define DOCSIS_MNG_TX_BUFFER_NUM        DOCSIS_MNG_TX_BD_NUM

#define DOCSIS_MNG_CPPI41_DMA_CHNUM     14

/* CPPI41 related MACROS used by CPGMAC */
#define CPPI4_DESC_TYPE_SHIFT_HOST      27
#define CPPI4_DESC_TYPE_HOST            16
#define CPPI4_PKT_TYPE_SHIFT            26
#define CPPI4_PKT_TYPE_GENERIC          6
#define CPPI4_PKT_RETPLCY_SHIFT         15
#define CPPI4_PKT_RETPLCY_FULL          0
#define CPPI4_DESC_LOC_SHIFT            14
#define CPPI4_DESC_LOC_OFFCHIP          0
#define CPPI4_PKT_RETQMGR_SHIFT         12
#define CPPI4_PKT_RETQ_SHIFT            0
#define CPPI4_DESC_TYPE_SHIFT_TD        27
#define CPPI4_DESC_TYPE_MASK_TD         (0x1F << CPPI4_DESC_TYPE_SHIFT_TD)
#define CPPI4_DESC_TYPE_TD              19

/* Only CPPI specified bytes need to be invalidated */
#define CPPI4_BD_LENGTH_FOR_CACHE       32

#define PAL_CPPI4_CACHE_INVALIDATE(addr, size)  dma_cache_inv ((unsigned long)(addr), (size))
#define PAL_CPPI4_CACHE_WRITEBACK(addr, size)   dma_cache_wback ((unsigned long)(addr), (size))
#define PAL_CPPI4_CACHE_WRITEBACK_INVALIDATE(addr, size)   dma_cache_wback_inv ((unsigned long)(addr), (size)) 

/* accumulator defines */
#define CPPI41_ACC_ENTRY_D              0 /* should be in the pal !!!*/
#define CPPI41_ACC_ENTRY_CD             1
#define CPPI41_ACC_ENTRY_ABCD           2

#define DOCSIS_MNG_ACC_ENTRY_TYPE   CPPI41_ACC_ENTRY_D
#define DOCSIS_MNG_ACC_MAXPAGENTRY  4
#define DOCSIS_MNG_ACC_ENTRY_SIZE   4 /* 4 bytes pointer */

#define DOCSIS_MNG_ACC_INTR_MODE    2
#define DOCSIS_MNG_ACC_INTR_DELAY   0
#define DOCSIS_MNG_ACC_STALL_AVOID  0
#define DOCSIS_MNG_ACC_TERM_MODE    0
#define DOCSIS_MNG_ACC_PAGE_COUNT   2

#define DOCSIS_MNG_RX_INTR_LINE             (AVALANCHE_INTD_BASE_INT + DOCSIS_MNG_ACCRX_INTD_VEC)

/* Protocol specific bits - US */
#define US_PSI_PRIVACY_OFF_BIT_MASK        0x00008000 /* bit 15 */
#define US_PSI_SFINDEX_MASK                0xFF000000 /* bits 31-24 */
#define US_PSI_SFINDEX_SHIFT               24

/* Protocol specific bits - DS */
#define DS_PROTOCOL_SPECIFIC_TS_PRESENT    0x00000020 /* bit 5 */
#define DS_PROTOCOL_SPECIFIC_TS_SIZE       4          /* Timestamp size in bytes */
#define DS_PROTOCOL_SPECIFIC_PORT_SHIFT    8          /* Port is located in bits 8..15 */
#define DS_PROTOCOL_SPECIFIC_PORT_SIZE     1          /* Port size in bytes */

DECLARE_WAIT_QUEUE_HEAD(docsis_mng_wait);
static int ref = 0;
static struct semaphore wr_sem;

/**
 *  Host descriptor structure. 
 * Note: This structure can represent either Host Packet Descriptor or Host 
 * Buffer Descriptor at a time, so validity of some fields depends on the 
 * type of descriptor.
 */
typedef volatile struct {
    /* hardware descriptor fields from this point */
    Uint32 descInfo;     /**< Desc type, proto specific word cnt, pkt len (valid only in Host PD)*/
    Uint32 tagInfo;      /**< Source tag (31:16), Dest Tag (15:0) (valid only in Host PD)*/
    Uint32 pktInfo;      /**< pkt err state, type, proto flags, return info, desc location */
    Uint32 buffLen;      /**< Number of valid data bytes in the buffer */
    Uint32 bufPtr;       /**< Pointer to the buffer associated with this descriptor */
    Ptr nextBDPtr;       /**< Pointer to the next buffer descriptor */
    Uint32 orgBuffLen;   /**< Original buffer size */
    Uint32 orgBufPtr;    /**< Original buffer pointer */
    Uint32 netInfoWord0; /**< Network stack private communications info (valid only in Host PD) */
    Uint32 netInfoWord1; /**< Network stack private communications info (valid only in Host PD) */
    
    /* Protocol specific data */
    Uint32 protocolSpec0;
    Uint32 protocolSpec1;
    Uint32 protocolSpec2;
    
    /* software only fields from this point */

} DocsisCppi4HostDesc;

static PAL_Handle palHandle;
static PAL_Cppi4QueueHnd fdQueueHdl;
static PAL_Cppi4QueueHnd rxQueueHdl;
static Ptr bdAreaAddrs;
static Ptr txbdAreaAddrs;
static Ptr bufferAreaAddrs;
static Ptr txbufferAreaAddrs;
static Cppi4Queue  fdQueue = {DOCSIS_MNG_QMGR,DOCSIS_MNG_FD_QNUM};
static Cppi4Queue  rxQueue = {DOCSIS_MNG_QMGR,DOCSIS_MNG_RX_QNUM};

static Cppi4Queue  txfdQueue = {DOCSIS_MNG_QMGR,DOCSIS_MNG_TX_FD_QNUM};
static Cppi4Queue  txQueue = {DOCSIS_MNG_QMGR,DOCSIS_MNG_TX_QNUM};
static PAL_Cppi4QueueHnd txfdQueueHdl;
static PAL_Cppi4QueueHnd txQueueHdl;
/* this is the place to set data offset */
/* Ilan <- consider to make it configurable parameter !!! */
static int txBufferOffset = 0; 

static PAL_Cppi4RxChHnd docsisMngChHnd;
static Cppi4RxChInitCfg chInfo;
static DocsisCppi4HostDesc* curBD4Read = NULL;
static Bool dataReady = False;

int debug_enable = 0;

#ifdef INIT_EXT_DMA

#define DOCSIS_MAC_PHY_CH_NUM   4
static PAL_Cppi4TxChHnd phyTxChHnd[DOCSIS_MAC_PHY_CH_NUM];

#endif

#ifdef POOLING
struct timer_list mngPoolTimer;
#endif

#ifdef _USE_ACC_LIST
static Uint8 accWrkArea[DOCSIS_MNG_ACC_MAXPAGENTRY*DOCSIS_MNG_ACC_PAGE_COUNT*4];
static Uint32 accCurPage = 0;
static Uint32 accCurBdOnPage = 0;
static Uint32 accDataListPtr[DOCSIS_MNG_ACC_PAGE_COUNT];
static Cppi4Queue  accRxQueue = {DOCSIS_MNG_QMGR,DOCSIS_MNG_RX_QNUM1};
static PAL_Cppi4QueueHnd accRxQueueHdl;
#endif

static DocsisCppi4HostDesc* docsisMngGetNextBd(void);
static Int32 HAL_DocsisCppiInit(void);

static int (* HAL_DocsisQosClass_ClassifyMacMngCB)(Uint8 *buf,Int16 *sfIndex,Int16 *phsIndex) = NULL;

irqreturn_t
docsisMngIsr(int irq, void *dev_id, struct pt_regs *regs)
{
    /* wake up (unblock) for reading data from userspace   */
    dataReady = True;
    wake_up_interruptible(&docsis_mng_wait);
    return IRQ_HANDLED;
}

/*
 * ref count the total number of opens (both read and write).
 * It makes the code simpler, by allocating all the needs of r/w
 * at open time and dealocating them (together) at the last close.
 */
static int
docsisMngOpen(struct inode *inode, struct file *file) 
{
    DocsisCppi4HostDesc *currBD = NULL;
    volatile static UINT32 bufferCount = 0;
    int r = 0;

    down(&wr_sem);
    printk(KERN_INFO "docsisMngOpen: ref %d\n", ref);
    if (++ref > 1) {
        up(&wr_sem);
        return r;
    }

    currBD = (DocsisCppi4HostDesc *)PAL_cppi4QueuePop(fdQueueHdl);
    if (currBD != NULL) {
        currBD->bufPtr = currBD->orgBufPtr;
        currBD->buffLen = 20;
        *((UINT32 *)currBD->bufPtr) = bufferCount;
        bufferCount++;
        PAL_cppi4QueuePush(rxQueueHdl, (Ptr)PAL_CPPI4_VIRT_2_PHYS((Uint32)currBD), 
            DOCSIS_MNG_QM_HDESC_SIZE_VAL, 0);
    }

#ifdef POOLING
    mngPoolTimer.expires = jiffies + HZ / 100;
    add_timer(&mngPoolTimer);
    printk(KERN_INFO "docsisMngOpen: start pool timer\n");
#else
    if (r = avalanche_intd_enable_interrupt(0, DOCSIS_MNG_ACC_RX_CHNUM) {
        printk(KERN_WARNING "docsisMngOpen: Unable to enable INTD interrupt \n");
        up(&wr_sem);
        return r;
    }

    /* register to the interrupt on the apdsp !!!*/
    if (r = request_irq(LNXINTNUM(DOCSIS_MNG_RX_INTR_LINE), docsisMngIsr, SA_INTERRUPT, "docsis mng", NULL)) {
        printk(KERN_WARNING "Failed to register the irq for docsis mng.\n");
        up(&wr_sem);
        return r;
    }
    printk(KERN_INFO "docsisMngOpen: set intr on line : %d\n , %d\n", LNXINTNUM(DOCSIS_MNG_RX_INTR_LINE),DOCSIS_MNG_RX_INTR_LINE);
#endif

    /* enable DMA channel */
    PAL_cppi4EnableRxChannel(docsisMngChHnd, 0);

#ifdef INIT_EXT_DMA
    /* enable MAC PHY TX DMA channel */
    for (r = 0; r < DOCSIS_MAC_PHY_CH_NUM; r++)
        PAL_cppi4EnableTxChannel(phyTxChHnd[r], 0);
#endif
    up(&wr_sem);
    return 0;
}

/* close function - called when the "file" /dev/docsis_mng is closed in userspace   */
static int
docsisMngRelease(struct inode *inode, struct file *file) 
{
    down(&wr_sem);
    printk(KERN_INFO "docsisMngRelease: ref %d\n", ref);
    if (--ref) {
        up(&wr_sem);
        return 0;
    }

#if 0 /* MG debug !!!! */
    /* should i close the channel and release the memory??? */
    PAL_cppi4DisableRxChannel(docsisMngChHnd, 0);

#ifdef INIT_EXT_DMA
    {
        int i;

        /* enable MAC PHY TX DMA channel */
        for (i = 0; i < DOCSIS_MAC_PHY_CH_NUM; i++)
            PAL_cppi4DisableTxChannel(phyTxChHnd[i], 0);
    }
#endif

#ifdef POOLING
    del_timer(&mngPoolTimer);
    printk(KERN_INFO "docsisMngOpen: del pool timer\n");
#else
    avalanche_intd_disable_interrupt(0, DOCSIS_MNG_ACC_RX_CHNUM);        

    /* free an interrupt */
    free_irq(LNXINTNUM(DOCSIS_MNG_RX_INTR_LINE), NULL);
#endif
    /* flush the rx queue */
#endif
    up(&wr_sem);
    return 0;
}

static
DocsisCppi4HostDesc* docsisMngGetNextBd()
{
#ifndef _USE_ACC_LIST
    DocsisCppi4HostDesc *curBD = (DocsisCppi4HostDesc *)PAL_cppi4QueuePop(rxQueueHdl);
    if (curBD == NULL)
        dataReady = False;  

    curBD = PAL_CPPI4_PHYS_2_VIRT(curBD);
    return(curBD);
#else
    /* all acc handling should be moved to the pal !!! */
    /* I don't want to duplicate it here    but i NEED to :( */
    Uint32 curBD;

    curBD = accDataListPtr[accCurPage] + accCurBdOnPage*DOCSIS_MNG_ACC_ENTRY_SIZE;
    if (debug_enable) {
        printk(KERN_INFO "docsisMngGetNextBd: curBd =%x \n",curBD);
        printk(KERN_INFO "docsisMngGetNextBd: accCurPage =%x, accCurBdOnPage=%x \n",accCurPage,accCurBdOnPage);
    }
    if (((Ptr)curBD) != NULL) {
        curBD = PAL_CPPI4_PHYS_2_VIRT(curBD);
        accCurBdOnPage++;
        return((DocsisCppi4HostDesc *)(curBD & QMGR_QUEUE_N_REG_D_DESC_ADDR_MASK));
    }
    /* last entry on the page */
    /* invalidate the working area page */
    PAL_CPPI4_CACHE_INVALIDATE(accDataListPtr[accCurPage], DOCSIS_MNG_ACC_MAXPAGENTRY * 4);

    accCurPage++;
    if (accCurPage>=DOCSIS_MNG_ACC_PAGE_COUNT)
        accCurPage = 0;
    accCurBdOnPage = 0;
    dataReady = False;
    /* decrement the interrupt counter page */
    avalanche_intd_set_interrupt_count (0, DOCSIS_MNG_ACC_RX_CHNUM, 1);
    /* clear the interrupt mask to get next interrupt */
    avalanche_intd_write_eoi (DOCSIS_MNG_ACC_RX_CHNUM);
    return(NULL);
#endif
}

/* read function called when from /dev/docsis_mng is read */
static ssize_t
docsisMngRead(struct file *file, char *buf, size_t count, loff_t *ppos) 
{
    static int  offset = 0; 
    Uint32      user_count = count; /* save original count specified by the user */
    Uint32      timestamp;          /* timestamp associated with the packet */
    Uint8       portId;             /* Port where message arrived */
    int         err;
    
    /* i can add a semaphore to protect the critical section, but should I ??? */
    /* is it possible to have two entities reading from this device ??? */

    if (debug_enable)
        printk(KERN_INFO "docsisMngRead: in with buffer size %d\n",count);
    /* if we are not in the middle of the message , go bring a new one*/
    if (curBD4Read == NULL) {
        /* get the message from queue */
        while ((curBD4Read = docsisMngGetNextBd()) == NULL) {
            if (file->f_flags & O_NONBLOCK)
                return(-EAGAIN);

            /* do i have an issue with race condition ??? dataReady*/
            /* need to change to wait_event_interruptible(docsis_mng_wait, condition); */
            if (debug_enable)
                printk(KERN_INFO "docsisMngRead: go sleep %x\n",(Uint32)curBD4Read);            
            wait_event_interruptible(docsis_mng_wait, (dataReady == True));
            if (debug_enable)
                printk(KERN_INFO "docsisMngRead: i am wake !\n");           
        }
        offset = 0;
    }
    /* invalidate the cache bd */
    PAL_CPPI4_CACHE_INVALIDATE(curBD4Read, CPPI4_BD_LENGTH_FOR_CACHE);
    if (debug_enable)
        printk(KERN_INFO "docsisMngRead: processing %x\n, data %x,size %d",
            (Uint32)curBD4Read, (Uint32)(PAL_CPPI4_PHYS_2_VIRT(curBD4Read->bufPtr)),
            curBD4Read->buffLen);           

    /* message is smaller than buffer */
    if (count >= (curBD4Read->buffLen - offset))
        count = (curBD4Read->buffLen - offset); 

    /* should i change the addr from phy to virt??? probably yes */
    err = copy_to_user(buf, (char*)PAL_CPPI4_PHYS_2_VIRT(curBD4Read->bufPtr) + offset, count);
    if (debug_enable)
        printk(KERN_INFO "docsisMngRead: copy to user %d bytes with offset %x\n",count,offset);         
    
    /* lets keep the message for the next read ???*/
    if (err != 0)
        return -EFAULT;

    /* if we done with the message */
    if (count == (curBD4Read->buffLen - offset)) {
        /* check if timestamp present */
        if (curBD4Read->protocolSpec0 & DS_PROTOCOL_SPECIFIC_TS_PRESENT) {
            if ((count + DS_PROTOCOL_SPECIFIC_TS_SIZE)> user_count) {
                if (debug_enable)
                    printk(KERN_WARNING "docsisMngRead: Not enough space to copy timestamp to user!\n");
            }
            else {
                /* get timestamp from BD and append it at end of packet */
                timestamp = curBD4Read->protocolSpec1;
                err = copy_to_user(buf + count, (char*)&timestamp, DS_PROTOCOL_SPECIFIC_TS_SIZE);
                if (debug_enable)
                    printk(KERN_INFO "docsisMngRead: Copy timestamp 0x%x to user (%d bytes) - %s.\n",
                           timestamp, DS_PROTOCOL_SPECIFIC_TS_SIZE, err ? "failed" : "ok");         
                if (err == 0)
                    count += DS_PROTOCOL_SPECIFIC_TS_SIZE;
            }
        }

        if ((count + DS_PROTOCOL_SPECIFIC_PORT_SIZE)> user_count) {
            if (debug_enable)
                printk(KERN_WARNING "docsisMngRead: Not enough space to copy port to user!\n");
        }
        else {
            portId = (curBD4Read->protocolSpec0 >> DS_PROTOCOL_SPECIFIC_PORT_SHIFT) + 1;
            err = copy_to_user(buf + count, (char*)&portId, DS_PROTOCOL_SPECIFIC_PORT_SIZE);
            if (debug_enable)
                printk(KERN_INFO "docsisMngRead: Copy port %d to user (%d bytes) at offset %d - %s.\n",
                       portId, DS_PROTOCOL_SPECIFIC_PORT_SIZE, count, err ? "failed" : "ok");         
            if (err == 0)
                count += DS_PROTOCOL_SPECIFIC_PORT_SIZE;
        }

        /* TBD clear protocol specific area !!!!*/
        /* invalidate the cache before returning the buffer to the hardware */
        PAL_CPPI4_CACHE_INVALIDATE(PAL_CPPI4_PHYS_2_VIRT(curBD4Read->orgBufPtr), curBD4Read->orgBuffLen);

        /* return the bd to the free queue */
        PAL_cppi4QueuePush(fdQueueHdl,(Ptr)PAL_CPPI4_VIRT_2_PHYS((Uint32)curBD4Read), DOCSIS_MNG_QM_HDESC_SIZE_VAL, 0);
        curBD4Read = NULL;

        {
            UINT32 status;
            volatile static Int32 packet_counter = 0;

            packet_counter++;   
            PAL_cppi4Control(palHandle,PAL_CPPI41_IOCTL_GET_QUEUE_ENTRY_COUNT,&fdQueue, &status);
            if (debug_enable)
                printk("Read : after packet %d , free %d\n", packet_counter,status);
        }
    } else
        offset += count; 

    if (debug_enable)
        printk(KERN_INFO "docsisMngRead: return %d bytes \n",count);
    return count;
}




static ssize_t docsisMngWrite(struct file *file, const char __user *buf, size_t count, loff_t *ppos)
{
    int retval;
    DocsisCppi4HostDesc *currBD = NULL;
    HalDocsisMng_t halMngData;
    struct CM_mng_t *macHdr;

    printk(KERN_INFO "Enter docsisMngWrite count %d\n", count);

    if (down_interruptible(&wr_sem))
        return -ERESTARTSYS;
    
    /* get the bd + buffer from the free queue */
    currBD = (DocsisCppi4HostDesc *) PAL_cppi4QueuePop(txfdQueueHdl);
    
    if(currBD==NULL)
    {
        printk(" %s can't allocate the free bd \n", __FUNCTION__);
        up(&wr_sem);
        return (-EAGAIN);
    }

    currBD = PAL_CPPI4_PHYS_2_VIRT(currBD);

    /* set the data pointer to copy from user */
    currBD->bufPtr = currBD->orgBufPtr + txBufferOffset;

    /*Copy the internal HAL data structure*/
    if (copy_from_user((void *)&halMngData, buf, count))
    {
        printk("%s : failed to copy HAL Mngmt data from user\n", __FUNCTION__);
        return -EFAULT;
    }

    printk(KERN_INFO "%s: encrFlag = %d msgBufLen=%d\n", __FUNCTION__, halMngData.encrFlag, halMngData.msgBufLen);

    if (halMngData.msgBufLen >= DOCSIS_MNG_TX_BUFFER_SIZE)
        return -EINVAL;     /* should be a whole message */

    /* Copy the management message buffer */
    if (copy_from_user((void *)(currBD->bufPtr), halMngData.msgBuf, halMngData.msgBufLen))
    {
        /* return the bd to the free queue */
        printk("%s : failed to copy management message buffer from user\n", __FUNCTION__);
        currBD->bufPtr = 0;
        PAL_CPPI4_CACHE_WRITEBACK(currBD, CPPI4_BD_LENGTH_FOR_CACHE);

        PAL_cppi4QueuePush (txfdQueueHdl,(Ptr) PAL_CPPI4_VIRT_2_PHYS((Uint32)currBD), 
                                                    DOCSIS_MNG_QM_HDESC_SIZE_VAL, 0);

        return -EINVAL; 
    }
    else 
    {
        /* set the bd fields and writeback the bd */
        currBD->buffLen = halMngData.msgBufLen;
        currBD->descInfo = PAL_CPPI4_HOSTDESC_DESC_TYPE_HOST << PAL_CPPI4_HOSTDESC_DESC_TYPE_SHIFT | halMngData.msgBufLen;
        
        if (!halMngData.encrFlag) 
            currBD->protocolSpec0 = US_PSI_PRIVACY_OFF_BIT_MASK;
        else
            currBD->protocolSpec0 = 0;

        macHdr = (struct CM_mng_t *)currBD->bufPtr;
        if (HAL_DocsisQosClass_ClassifyMacMngCB)
        {
            Int16 sfIndex;
            Int16 phsIndex;

            /* Classifier exceptions - don't classify */
            if(!((macHdr->type == MAC_MNGT_MSG_TYPE_RNG_REQ) ||
                 (macHdr->type == MAC_MNGT_MSG_TYPE_REG_REQ) ||
                 (macHdr->type == MAC_MNGT_MSG_TYPE_REG_RSP) ||
                 (macHdr->type == MAC_MNGT_MSG_TYPE_REG_ACK) ||
                 (macHdr->type == MAC_MNGT_MSG_TYPE_INIT_RNG_REQ)   ||
                 (macHdr->type == MAC_MNGT_MSG_TYPE_B_INIT_RNG_REQ) ||
                 (macHdr->type == MAC_MNGT_MSG_TYPE_REG_REQ_MP) ||
                 (macHdr->type == MAC_MNGT_MSG_TYPE_REG_RSP_MP) || 
                 (macHdr->type == MAC_MNGT_MSG_TYPE_DPV_RSP))) 
            {
                HAL_DocsisQosClass_ClassifyMacMngCB((Uint8 *)currBD->bufPtr, &sfIndex,  &phsIndex);
                currBD->protocolSpec0 |= ((((Uint32) sfIndex) << US_PSI_SFINDEX_SHIFT));
            }
         }

        /* DPV-RSP is sent over user-specified service flow if so requested. */
        if ((macHdr->type == MAC_MNGT_MSG_TYPE_DPV_RSP) && halMngData.forceSF)
        {
            printk(KERN_INFO "%s: Forcing SF index 0x%x for DPV-RSP\n", __FUNCTION__, halMngData.sfIndex);
            currBD->protocolSpec0 |= (halMngData.sfIndex << US_PSI_SFINDEX_SHIFT);
        }

        printk(KERN_INFO "%s: currBD->protocolSpec0 = %x currBD->buffLen=%d\n", __FUNCTION__, currBD->protocolSpec0, currBD->buffLen);

		PAL_CPPI4_CACHE_WRITEBACK(currBD, CPPI4_BD_LENGTH_FOR_CACHE);

        
        /* writeback the data */
        PAL_CPPI4_CACHE_WRITEBACK(currBD->bufPtr, halMngData.msgBufLen);

        /* push the bd to the management tx queue */
        /* return policy in the bd should be set to auto recycling */
        PAL_cppi4QueuePush (txQueueHdl,(Ptr) PAL_CPPI4_VIRT_2_PHYS((Uint32)currBD), 
                                                                DOCSIS_MNG_QM_HDESC_SIZE_VAL, halMngData.msgBufLen);
    }

    *ppos += count;
    retval = count;

    up(&wr_sem);

    return retval;
}

#ifdef POOLING
void
docsisTimerFunc(unsigned long data)
{
    UINT32 status;
    
    PAL_cppi4Control (palHandle,PAL_CPPI41_IOCTL_GET_QUEUE_PEND_STATUS,&rxQueue, &status);
    
    if (status) {
        UINT32 tmp;
        PAL_cppi4Control (palHandle,PAL_CPPI41_IOCTL_GET_QUEUE_PEND_STATUS,&fdQueue, &tmp);
        if (debug_enable)
            printk("Timer : waiting %d packets\n", tmp);

        if (debug_enable)
            printk(KERN_INFO "docsisTimerFunc : wake the read \n"); 
        /* wake up (unblock) for reading data from userspace   */
        dataReady = True;
        wake_up_interruptible(&docsis_mng_wait);
    }

//  mngPoolTimer.expires = jiffies + HZ;
    mngPoolTimer.expires = jiffies + HZ/100;
    add_timer(&mngPoolTimer);
}
#endif

static unsigned int
docsisMngPoll(struct file *file, poll_table * wait)
{
    if (debug_enable)
        printk("docsisMngPoll: curBD4Read %x\n", (Uint32)curBD4Read);
    poll_wait(file, &docsis_mng_wait, wait);
    if (debug_enable)
        printk("docsisMngPoll: post poll_wait curBD4Read %x\n", (Uint32)curBD4Read);
    if (curBD4Read || (curBD4Read = docsisMngGetNextBd())) {
        if (debug_enable)
            printk("docsisMngPoll: before POLLIN return curBD4Read %x\n", (Uint32)curBD4Read);
        return POLLIN | POLLRDNORM;
    }
    return 0;
}

struct file_operations docsis_mng_fops = {
    .owner   = THIS_MODULE,       
    .llseek  = NULL,              
    .read    = docsisMngRead,   
    .write   = docsisMngWrite,              
    .poll    = docsisMngPoll,  
    .ioctl   = NULL,  
    .open    = docsisMngOpen,   
    .release = docsisMngRelease,
};

static int
mng_read_proxy(char *buf, char **start, off_t offset, int count, int *eof, void *data)
{
    int len = 0;

    len += sprintf(buf + len, "debug state %d \n",debug_enable);
    debug_enable = 0;
    return(len);
}

static void
cleanup(int level)
{
    switch (level) {
    default:
    case 4:
        PAL_cppi4DeallocDesc(palHandle, DOCSIS_MNG_CPPI41_RX_QMGR, bdAreaAddrs);
    case 3:
        PAL_cppi4QueueClose(palHandle, rxQueueHdl);
    case 2:
        PAL_cppi4QueueClose(palHandle, fdQueueHdl);
    case 1:
        PAL_cppi4Exit(palHandle, NULL);
        break;
    }
}

/* initialize module (and interrupt) */
static int __init docsisMngInit (void) 
{
    int i,cnt;
    PAL_Result retCode;
    DocsisCppi4HostDesc *currBD;
    Ptr curBuffer;

    init_MUTEX(&wr_sem);
    if (debug_enable)
        printk(KERN_INFO "docsisMngInit: initializing module\n");
    
    /* get a handle to the PAL structure */
    if ((palHandle =  PAL_cppi4Init(NULL, NULL)) == NULL) {
        printk(KERN_WARNING "docsisMngInit: PAL CPPI is not initialized yet \n");
        return(-EAGAIN); /* what should i return ???*/
    }   

    printk(KERN_INFO "docsisMngInit: pal handle %x\n", (Uint32)palHandle);
    
    /*  open a queue objects one for free descriptor queue
     *  and one for rx queue 
     */ 
    if ((fdQueueHdl = PAL_cppi4QueueOpen(palHandle, fdQueue)) == NULL) {
        printk(KERN_WARNING "docsisMngInit: Failed to open fdQueue\n");
        cleanup(1);
        return -1;
    }
    if ((rxQueueHdl = PAL_cppi4QueueOpen(palHandle, rxQueue)) == NULL) {
        printk(KERN_WARNING "docsisMngInit: Failed to open rxQueueHdl\n");
        cleanup(2);
        return -1;
    }

    /*
     * get a pointer to the BD region 
     * the region is already preallocated in the PAL init so just 
     * get a pointer to the bd area 
     */
    if ((bdAreaAddrs = PAL_cppi4AllocDesc(palHandle, DOCSIS_MNG_CPPI41_RX_QMGR, DOCSIS_MNG_RX_BD_NUM, DOCSIS_MNG_RX_BD_SIZE)) == NULL) {
        printk(KERN_WARNING "docsisMngInit: Failed to allocate Rx descriptor\n");
        cleanup(3);
        return -1;
    }

    printk(KERN_INFO "docsisMngInit: bd area start address %x\n",(Uint32)bdAreaAddrs);

    /* allocate a memory for the buffers , is it better to get it one by one?*/
    if ((retCode = PAL_osMemAlloc(0, DOCSIS_MNG_RX_BUFFER_NUM * DOCSIS_MNG_RX_BUFFER_SIZE, 0, (Ptr *)&bufferAreaAddrs)) != PAL_SOK) {
        printk(KERN_WARNING "docsisMngInit:Failed to allocate memory for RX buffer queue");
        cleanup(4);
        return -1;
    }
    if (debug_enable)
        printk(KERN_INFO "docsisMngInit : buffer area start address %x\n",(Uint32)bufferAreaAddrs);
 
    PAL_osMemSet(bufferAreaAddrs,0,DOCSIS_MNG_RX_BUFFER_NUM * DOCSIS_MNG_RX_BUFFER_SIZE);

    /* create a dedicated free bd/buffers queue for this channel */
    /* "Slice" BD's one-by-one from the chunk, allocate a buffer and token, */
    currBD = (DocsisCppi4HostDesc *)bdAreaAddrs;
    curBuffer = bufferAreaAddrs;

    /* need to consider the number of actually allocated buffers !!!!*/
    /* we don't really need 32 pair for management channel, do we? */
    for (cnt = 0; cnt < DOCSIS_MNG_RX_BUFFER_NUM; cnt++) {
        /* Update the hardware descriptor */
        currBD->descInfo    = ((CPPI4_DESC_TYPE_HOST << CPPI4_DESC_TYPE_SHIFT_HOST));
        currBD->tagInfo     = 0;
        currBD->pktInfo     = (CPPI4_PKT_TYPE_GENERIC << CPPI4_PKT_TYPE_SHIFT) 
                                | (CPPI4_PKT_RETPLCY_FULL << CPPI4_PKT_RETPLCY_SHIFT)
                                | (CPPI4_DESC_LOC_OFFCHIP << CPPI4_DESC_LOC_SHIFT)
                                | (0 << CPPI4_PKT_RETQMGR_SHIFT)
                                | (0 << CPPI4_PKT_RETQ_SHIFT);
        currBD->buffLen     = 0;
        currBD->bufPtr      = 0;
        currBD->nextBDPtr   = 0;
        currBD->orgBuffLen  = DOCSIS_MNG_RX_BUFFER_SIZE;
        currBD->orgBufPtr   = PAL_CPPI4_VIRT_2_PHYS(curBuffer);

        PAL_CPPI4_CACHE_WRITEBACK(currBD, CPPI4_BD_LENGTH_FOR_CACHE);

        PAL_cppi4QueuePush (fdQueueHdl,(Ptr) PAL_CPPI4_VIRT_2_PHYS((Uint32)currBD), 
                                                    DOCSIS_MNG_QM_HDESC_SIZE_VAL, 0);
    
        currBD = (DocsisCppi4HostDesc*) ((Uint32)currBD + DOCSIS_MNG_RX_BD_SIZE);
        curBuffer = (Ptr) ((Uint32)curBuffer + DOCSIS_MNG_RX_BUFFER_SIZE);
    }

    /* configure the DMA channel to use this queue*/

    chInfo.chNum = DOCSIS_MNG_CPPI41_DMA_CHNUM;
	chInfo.dmaNum = 1;
    chInfo.defDescType=1; /* host descriptor */
    chInfo.sopOffset=0;
    chInfo.rxCompQueue=rxQueue;
    chInfo.u.hostPktCfg.fdbQueue[0] = fdQueue;  
    chInfo.u.hostPktCfg.fdbQueue[1] = fdQueue;  
    chInfo.u.hostPktCfg.fdbQueue[2] = fdQueue;  
    chInfo.u.hostPktCfg.fdbQueue[3] = fdQueue;  

#ifdef _USE_ACC_LIST
    /* don't forget to init the accumulator !!!! */
    chInfo.accuCfg.accChanNum = DOCSIS_MNG_ACC_RX_CHNUM;
    chInfo.accuCfg.listBase = (Ptr) PAL_CPPI4_VIRT_2_PHYS(accWrkArea); /*malloc or static allocation ???*/
    chInfo.accuCfg.maxPageEntry = DOCSIS_MNG_ACC_MAXPAGENTRY;
    chInfo.accuCfg.pacingTickCnt = DOCSIS_MNG_ACC_INTR_DELAY;
    chInfo.accuCfg.pacingMode = DOCSIS_MNG_ACC_INTR_MODE; /* delay since first new packet */
    chInfo.accuCfg.stallAvoidance = DOCSIS_MNG_ACC_STALL_AVOID;
    chInfo.accuCfg.listCountMode = DOCSIS_MNG_ACC_TERM_MODE; /* NULL Terminate Mode - The last list entry is used to store a NULL pointer record <BR> */
    chInfo.accuCfg.listEntrySize = DOCSIS_MNG_ACC_ENTRY_TYPE;
    chInfo.accuCfg.maxPageCnt = DOCSIS_MNG_ACC_PAGE_COUNT;

    PAL_osMemSet(accWrkArea,0,sizeof(accWrkArea));

    /* invalidate the page !!!*/
    accCurPage = 0;
    accDataListPtr[0] = (Uint32) accWrkArea;
    accDataListPtr[1] = (Uint32) accWrkArea + DOCSIS_MNG_ACC_MAXPAGENTRY*4 ;
    accRxQueueHdl = PAL_cppi4QueueOpen (palHandle,accRxQueue);
#endif

    docsisMngChHnd = PAL_cppi4RxChOpen (palHandle, &chInfo, NULL);
    if (docsisMngChHnd == NULL) {
        if (debug_enable)
            printk(KERN_INFO "Can't open the rx ch %x \n", (Uint32)(&chInfo));
        PAL_osMemFree(0,bufferAreaAddrs,DOCSIS_MNG_RX_BUFFER_NUM * DOCSIS_MNG_RX_BUFFER_SIZE); 
        PAL_cppi4QueueClose (palHandle, fdQueueHdl);
        PAL_cppi4QueueClose (palHandle, rxQueueHdl);
        PAL_cppi4RxChClose(docsisMngChHnd,0);   
        return(-1); /* what should i return ???*/
    }
    if (debug_enable)
        printk(KERN_INFO "PAL_cppi4RxChOpen ret %x\n", (Uint32)docsisMngChHnd);

#ifdef INIT_EXT_DMA
    {
        volatile Cppi4TxChInitCfg txChinfo;
        static Cppi4Queue  tdQueue = {0,0}; /* we don't have teardown one so don't care */
        int i;

        txChinfo.chNum = 9;               
		txChinfo.dmaNum = 1;              
        txChinfo.tdQueue = tdQueue;         

        for (i = 0; i < DOCSIS_MAC_PHY_CH_NUM; i++) {
            phyTxChHnd[i] = PAL_cppi4TxChOpen (palHandle , (Cppi4TxChInitCfg *)(&txChinfo), NULL);
            /* don't care about free the resources it is patch */
            if (phyTxChHnd[i] == NULL) {
                if (debug_enable)
                    printk(KERN_INFO "Cant open TX channel 0 \n");
                return(-1);
            }
            /* no need to change the rest of the params except the channel number */
            txChinfo.chNum += 1;    
        }
    }
#endif

#ifdef POOLING
    init_timer(&mngPoolTimer);
    mngPoolTimer.function = docsisTimerFunc;
    mngPoolTimer.expires = 0;
#endif
    if (debug_enable)
        printk(KERN_INFO "Initializing the DOCSIS CPPI Resources\n");
    if (HAL_DocsisCppiInit() != 0) {
        if (debug_enable)
            printk(KERN_INFO "Failed to initialize the DOCSIS CPPI resources \n");
        return(-1);
    }


    /* ******************* init TX part of the driver ******************* */
    /*  open a queue objects one for free descriptor queue
     *  and one for tx queue 
     */ 
    if ((txfdQueueHdl = PAL_cppi4QueueOpen(palHandle, txfdQueue)) == NULL) {
        printk(KERN_WARNING "docsisMngInit: Failed to open fdQueue\n");
        cleanup(1);
        return -1;
    }
    if ((txQueueHdl = PAL_cppi4QueueOpen(palHandle, txQueue)) == NULL) {
        printk(KERN_WARNING "docsisMngInit: Failed to open rxQueueHdl\n");
        cleanup(2);
        return -1;
    }

    /*
     * get a pointer to the BD region 
     * the region is already preallocated in the PAL init so just 
     * get a pointer to the bd area 
     */
    if ((txbdAreaAddrs = PAL_cppi4AllocDesc(palHandle, 
                DOCSIS_MNG_QMGR, DOCSIS_MNG_TX_BD_NUM, DOCSIS_MNG_TX_BD_SIZE)) == NULL) 
    {
        printk(KERN_WARNING "docsisMngInit: Failed to allocate TX descriptor\n");
        cleanup(3);
        return -1;
    }

    printk(KERN_INFO "docsisMngInit: TX bd area start address %x\n",(Uint32)txbdAreaAddrs);

    /* allocate a memory for the buffers , is it better to get it one by one?*/
    if ((retCode = PAL_osMemAlloc(0, DOCSIS_MNG_TX_BUFFER_NUM * DOCSIS_MNG_TX_BUFFER_SIZE, 
                                0, (Ptr *)&txbufferAreaAddrs)) != PAL_SOK) 
    {
        printk(KERN_WARNING "docsisMngInit:Failed to allocate memory for TX buffer queue");
        cleanup(4);
        return -1;
    }
    
    if (debug_enable) printk(KERN_INFO "docsisMngInit : buffer area start address %x\n",(Uint32)bufferAreaAddrs);
 
    PAL_osMemSet(txbufferAreaAddrs,0,DOCSIS_MNG_TX_BUFFER_NUM * DOCSIS_MNG_TX_BUFFER_SIZE);

    /* create a dedicated free bd/buffers queue for this channel */
    /* "Slice" BD's one-by-one from the chunk, allocate a buffer and token, */
    currBD = (DocsisCppi4HostDesc *)txbdAreaAddrs;
    curBuffer = txbufferAreaAddrs;

    /* need to consider the number of actually allocated buffers !!!!*/
    /* we don't really need 32 pair for management channel, do we? */
    for (cnt = 0; cnt < DOCSIS_MNG_TX_BUFFER_NUM; cnt++) {
        /* Update the hardware descriptor */
        currBD->descInfo    = ((CPPI4_DESC_TYPE_HOST << CPPI4_DESC_TYPE_SHIFT_HOST));
        currBD->tagInfo     = 0;
        currBD->pktInfo     = (CPPI4_PKT_TYPE_GENERIC << CPPI4_PKT_TYPE_SHIFT) 
                                | (CPPI4_PKT_RETPLCY_FULL << CPPI4_PKT_RETPLCY_SHIFT)
                                | (CPPI4_DESC_LOC_OFFCHIP << CPPI4_DESC_LOC_SHIFT)
                                | (DOCSIS_MNG_QMGR << CPPI4_PKT_RETQMGR_SHIFT)
                                | (DOCSIS_MNG_TX_FD_QNUM << CPPI4_PKT_RETQ_SHIFT);
        currBD->buffLen     = 0;
        currBD->bufPtr      = 0;
        currBD->nextBDPtr   = 0;
        currBD->orgBuffLen  = DOCSIS_MNG_TX_BUFFER_SIZE;
        currBD->orgBufPtr   = PAL_CPPI4_VIRT_2_PHYS(curBuffer);

        PAL_CPPI4_CACHE_WRITEBACK(currBD, CPPI4_BD_LENGTH_FOR_CACHE);

        PAL_cppi4QueuePush (txfdQueueHdl,(Ptr) PAL_CPPI4_VIRT_2_PHYS((Uint32)currBD), 
                                                    DOCSIS_MNG_QM_HDESC_SIZE_VAL, 0);
    
        currBD = (DocsisCppi4HostDesc*) ((Uint32)currBD + DOCSIS_MNG_TX_BD_SIZE);
        curBuffer = (Ptr) ((Uint32)curBuffer + DOCSIS_MNG_TX_BUFFER_SIZE);
    }

    if (debug_enable) printk(KERN_INFO "TX init done \n");
    /* ******************* Done ******************* */


    i = register_chrdev(DOCSIS_MNG_MAJOR, DOCSIS_MNG_NAME, &docsis_mng_fops);
    if (i != 0)
        return - EIO;

    create_proc_read_entry("mng", 0, NULL, mng_read_proxy, NULL);
    return 0;
}

/* close and cleanup module */
static void __exit docsisMngCleanup (void) 
{
    if (debug_enable)
        printk(KERN_INFO "cleaning up module\n");
    unregister_chrdev(DOCSIS_MNG_MAJOR, DOCSIS_MNG_NAME);

    PAL_osMemFree(0, bufferAreaAddrs, DOCSIS_MNG_RX_BUFFER_NUM * DOCSIS_MNG_RX_BUFFER_SIZE); 
    PAL_cppi4QueueClose (palHandle, fdQueueHdl);
    PAL_cppi4QueueClose (palHandle, rxQueueHdl);
    PAL_cppi4RxChClose(docsisMngChHnd,0);   
    /* should i support DMA teardown */
}

/***************************************************************************************************************/
/***************************************************************************************************************/
/***************************************************************************************************************/
/***************************************************************************************************************/
/*                                     Internal DDOCSIS CPPI code                                              */
/***************************************************************************************************************/
/***************************************************************************************************************/
/***************************************************************************************************************/
/***************************************************************************************************************/

void testWriteRegister(Uint32 addr, Uint32 value, Bool printEn)
{
    (*(volatile Uint32*)(addr)) = value;
    if (printEn)
        if (debug_enable)
            printk(KERN_INFO "Write Virtual: [0x%08X] <-- 0x%08x ; Write Physical: [0x%08X] <-- 0x%08x\n", addr, value, IO_VIRT2PHY(addr), value);
}

/**********************************************************************
 *  \brief HAL_DocsisCppiBufferMngInit
 * 
 *  This function initialize the DOCSIS Buffer manager.
 *  Only the DS MAC uses the DOCSIS Buffer Manager. There is 3 buffer
 *  pools in the DOCSIS buffer manager, however currently the DS uses only
 *  one pool, which contains 64 buffers of 512 bytes each one.
 *
 *  @return  RCODE_SUCCESS
 *                     
 */
static Int32
HAL_DocsisCppiBufferMngInit(void)
{
    Uint32 regContent = 0;
    Uint32 regAddress = 0;
    Uint32 freeBufPool0CtrReg = HAL_MAC_BASE_ADDRESS+DOCSIS_BMGR_FREE_BUF_POOL_N_CTRL(DS_FREE_BUFFER_POOL_INDEX);

    if (debug_enable)
        printk(KERN_INFO "\n*****HAL_DocsisCppiBufferMngInit:\n");

    /* Set the base address of buffer pool #0 */
    regContent = DOCSIS_MAC_DS_INTERNAL_RAM_BASE_ADDR;
    regAddress = HAL_MAC_BASE_ADDRESS+DOCSIS_BMGR_FREE_BUF_POOL_N_ADDR(DS_FREE_BUFFER_POOL_INDEX);

    /*HAL_SetMacReg32( DOCSIS_BMGR_FREE_BUF_POOL_N_ADDR(DS_FREE_BUFFER_POOL_INDEX), DOCSIS_MAC_DS_INTERNAL_RAM_BASE_ADDR );*/
    /*(*(volatile Uint32*)(regAddress))= regContent;*/
    testWriteRegister(regAddress, regContent, True);

    /* Initialize Pool #0 */
    regContent = 0;
    BMGR_SET_FREE_POOL_SIZE(regContent,DS_FREE_BUFFER_POOL_SIZE);
    BMGR_SET_FREE_POOL_BUFF_SIZE(regContent,DS_FREE_BUFFER_POOL_BUFFS_SIZE);
    BMGR_SET_FREE_POOL_REF_CNT(regContent,False);
    BMGR_SET_FREE_POOL_ENABLE(regContent,True);

    /*HAL_SetMacReg32( freeBufPool0CtrReg, regContent );*/
    /*(*(volatile Uint32*)(freeBufPool0CtrReg))= regContent;*/
    testWriteRegister(freeBufPool0CtrReg, regContent, True);
    if (debug_enable)
        printk(KERN_INFO "freeBufPool0CtrReg (virtual) = 0x%08X\n",freeBufPool0CtrReg);
    if (debug_enable)
        printk(KERN_INFO "freeBufPool0CtrReg (physical) = 0x%08X\n", IO_VIRT2PHY(freeBufPool0CtrReg));
    return (0);
}

/**********************************************************************
 *  \brief HAL_DocsisCppiQmngDescInit
 * 
 *  This function initialize the DS and US Embedded descriptors.
 *  It 
 *  Please note that the US FW is responsible to initialize the
 *  monolithic descriptors and push it to the free pools(queue #59)
 *  
 *
 *  @return  RCODE_SUCCESS
 *                     
 */
static Int32
HAL_DocsisCppiQmngDescInit(Uint32 usRegionBaseAddr)
{
    Uint32 i;
    volatile Uint32 baseAddr;

    if (debug_enable)
        printk(KERN_INFO "\n*****HAL_DocsisCppiQmngDescInit:\n");

    /*******************************************************/
    /* Downstream Descriptors - Internal DOCSIS MAC memory */
    /*******************************************************/
    memset((void *)DS_EMBEDDED_DESC_RAM_BASE_ADDR_HOST,0,DS_EMBEDDED_DESC_RAM_DESC_SIZE*DS_NUM_OF_EMBEDDE_DESCRIPTORS);
    if (debug_enable)
        printk(KERN_INFO "Initializing Downstream Descriptors\n");
    if (debug_enable)
        printk(KERN_INFO "Writing 0 to Virtual [0x%08X] (Physical: [0x%08X]), size = %d \n", DS_EMBEDDED_DESC_RAM_BASE_ADDR_HOST, IO_VIRT2PHY(DS_EMBEDDED_DESC_RAM_BASE_ADDR_HOST), DS_EMBEDDED_DESC_RAM_DESC_SIZE*DS_NUM_OF_EMBEDDE_DESCRIPTORS);
    if (debug_enable)
        printk(KERN_INFO "(Printing only first BD)\n");

    /* Initialize Embedded Descriptors for downstream channel in internal memory */
    for (i = 0; i < DS_NUM_OF_EMBEDDE_DESCRIPTORS; i++) {
        /*baseAddr = DS_EMBEDDED_DESC_RAM_BASE_ADDR_OFFSET + i*DS_EMBEDDED_DESC_RAM_DESC_SIZE;*/
        baseAddr = DS_EMBEDDED_DESC_RAM_BASE_ADDR_HOST + i*DS_EMBEDDED_DESC_RAM_DESC_SIZE;

        /* Set the PS size to 5 words */
        /*HAL_SetMacReg32( baseAddr + DS_EMBEDDED_DESC_WORD_0_OFFSET, DS_EMBEDDED_DESC_WORD_0_DEFAULT_VAL );*/
        /*(*(volatile Uint32*)(baseAddr + DS_EMBEDDED_DESC_WORD_0_OFFSET))= DS_EMBEDDED_DESC_WORD_0_DEFAULT_VAL;*/
        testWriteRegister((baseAddr + DS_EMBEDDED_DESC_WORD_0_OFFSET),DS_EMBEDDED_DESC_WORD_0_DEFAULT_VAL, (i==0));

        /*HAL_SetMacReg32( baseAddr + DS_EMBEDDED_DESC_WORD_2_OFFSET, DS_EMBEDDED_DESC_WORD_2_DEFAULT_VAL );*/
        /*(*(volatile Uint32*)(baseAddr + DS_EMBEDDED_DESC_WORD_2_OFFSET))= DS_EMBEDDED_DESC_WORD_2_DEFAULT_VAL;*/
        testWriteRegister((baseAddr + DS_EMBEDDED_DESC_WORD_2_OFFSET),DS_EMBEDDED_DESC_WORD_2_DEFAULT_VAL, (i==0));

        /* Initialize Packet Descriptors Protocol Specific Region */
        /*HAL_SetMacReg32( baseAddr + DS_EMBEDDED_DESC_BPI_CMD_OFFSET, DS_EMBEDDED_DESC_BPI_CMD_TEMPLATE );*/
        /*(*(volatile Uint32*)(baseAddr + DS_EMBEDDED_DESC_BPI_CMD_OFFSET))= DS_EMBEDDED_DESC_BPI_CMD_TEMPLATE;*/
        testWriteRegister((baseAddr + DS_EMBEDDED_DESC_BPI_CMD_OFFSET),DS_EMBEDDED_DESC_BPI_CMD_TEMPLATE, (i==0));

        /*HAL_SetMacReg32( baseAddr + DS_EMBEDDED_DESC_CRC_CMD_OFFSET, DS_EMBEDDED_DESC_CRC_CMD_TEMPLATE );*/
        /*(*(volatile Uint32*)(baseAddr + DS_EMBEDDED_DESC_CRC_CMD_OFFSET))= DS_EMBEDDED_DESC_CRC_CMD_TEMPLATE;*/
        testWriteRegister((baseAddr + DS_EMBEDDED_DESC_CRC_CMD_OFFSET),DS_EMBEDDED_DESC_CRC_CMD_TEMPLATE, (i==0));

        /*HAL_SetMacReg32( baseAddr + DS_EMBEDDED_DESC_PHS_CMD_OFFSET, DS_EMBEDDED_DESC_PHS_CMD_TEMPLATE );*/
        /*(*(volatile Uint32*)(baseAddr + DS_EMBEDDED_DESC_PHS_CMD_OFFSET))= DS_EMBEDDED_DESC_PHS_CMD_TEMPLATE;*/
        testWriteRegister((baseAddr + DS_EMBEDDED_DESC_PHS_CMD_OFFSET),DS_EMBEDDED_DESC_PHS_CMD_TEMPLATE, (i==0));

        /*HAL_SetMacReg32( baseAddr + DS_EMBEDDED_DESC_RX_FIFO_0_CMD_OFFSET, DS_EMBEDDED_DESC_RX_FIFO_0_CMD_TEMPLATE );*/
        /*(*(volatile Uint32*)(baseAddr + DS_EMBEDDED_DESC_RX_FIFO_0_CMD_OFFSET))= DS_EMBEDDED_DESC_RX_FIFO_0_CMD_TEMPLATE;*/
        testWriteRegister((baseAddr + DS_EMBEDDED_DESC_RX_FIFO_0_CMD_OFFSET),DS_EMBEDDED_DESC_RX_FIFO_0_CMD_TEMPLATE, (i==0));

        /*HAL_SetMacReg32( baseAddr + DS_EMBEDDED_DESC_RX_FIFO_1_CMD_OFFSET, DS_EMBEDDED_DESC_RX_FIFO_1_CMD_TEMPLATE);*/
        /*(*(volatile Uint32*)(baseAddr + DS_EMBEDDED_DESC_RX_FIFO_1_CMD_OFFSET))= DS_EMBEDDED_DESC_RX_FIFO_1_CMD_TEMPLATE;*/
        testWriteRegister((baseAddr + DS_EMBEDDED_DESC_RX_FIFO_1_CMD_OFFSET),DS_EMBEDDED_DESC_RX_FIFO_1_CMD_TEMPLATE, (i==0));

    }

    /* The US FW is responsible for the Monolithic Descriptors in 
       DTCM memory Initialization and create the free descriptors queue */

    /* Initialize Embedded Descriptors for upstream Rx channel in DDR2 memory */
    /* Fill the tree first words of the embedded descriptors */
    /* LIAT - NEED TO alloc  memory for US embedded descriptors */

    /************************************************************/
    /* Upstream Descriptors - External memory allocated by Host */
    /************************************************************/

    /*                                                   64               *          512           = 32KB   */
    /*memset(US_EMBEDDED_DESC_DDR_BASE_ADDR,0,US_EMBEDDED_DESC_RAM_DESC_SIZE*US_NUM_OF_EMBEDDE_DESC);*/
    memset((void *)usRegionBaseAddr,0,US_EMBEDDED_DESC_RAM_DESC_SIZE*US_NUM_OF_EMBEDDE_DESC);
    if (debug_enable)
        printk(KERN_INFO "Initializing Upstream Descriptors\n");
    if (debug_enable)
        printk(KERN_INFO "Writing 0 to Virtual [0x%08X] (Physical: [0x%08X]), size = %d \n", (Uint32)usRegionBaseAddr, (Uint32)(PAL_CPPI4_VIRT_2_PHYS(usRegionBaseAddr)), (Uint32)(US_EMBEDDED_DESC_RAM_DESC_SIZE*US_NUM_OF_EMBEDDE_DESC));
    if (debug_enable)
        printk(KERN_INFO "(Printing only first BD)\n");
    for (i = 0; i < US_NUM_OF_EMBEDDE_DESC; i++) {
        baseAddr = usRegionBaseAddr + i*US_EMBEDDED_DESC_RAM_DESC_SIZE;
        (*(volatile Uint32 *)(baseAddr)) = US_EMBEDDED_DESC_WORD_0_DEFAULT_VAL;
        if (i==0)
            if (debug_enable)
                printk(KERN_INFO "Virtual: [0x%08X] <-- 0x%08x ; Physical: [0x%08X] <-- 0x%08x\n", (Uint32)baseAddr, (Uint32)US_EMBEDDED_DESC_WORD_0_DEFAULT_VAL, (Uint32)(PAL_CPPI4_VIRT_2_PHYS(baseAddr)), (Uint32)US_EMBEDDED_DESC_WORD_0_DEFAULT_VAL);

        (*(volatile Uint32 *)(baseAddr+ 0x4)) = US_EMBEDDED_DESC_WORD_1_DEFAULT_VAL;
        if (i==0)
            if (debug_enable)
                printk(KERN_INFO "Virtual: [0x%08X] <-- 0x%08x ; Physical: [0x%08X] <-- 0x%08x\n", baseAddr+0x4, US_EMBEDDED_DESC_WORD_1_DEFAULT_VAL, PAL_CPPI4_VIRT_2_PHYS(baseAddr), US_EMBEDDED_DESC_WORD_1_DEFAULT_VAL);

        (*(volatile Uint32 *)(baseAddr+ 0x8)) = US_EMBEDDED_DESC_WORD_2_DEFAULT_VAL;
        if (i==0)
            if (debug_enable)
                printk(KERN_INFO "Virtual: [0x%08X] <-- 0x%08x ; Physical: [0x%08X] <-- 0x%08x\n", baseAddr+0x8, US_EMBEDDED_DESC_WORD_2_DEFAULT_VAL, PAL_CPPI4_VIRT_2_PHYS(baseAddr), US_EMBEDDED_DESC_WORD_2_DEFAULT_VAL);

        /* Note: this is done in external memory, which is a cachable memory-area. */
        /* Invoke Cache Writeback - make sure that the values are written to mem.  */
        PAL_CPPI4_CACHE_WRITEBACK(baseAddr, US_EMBEDDED_DESC_RAM_DESC_SIZE);
        if (i==0)
            if (debug_enable)
                printk(KERN_INFO "PAL_CPPI4_CACHE_WRITEBACK on Virtual [0x%08X] (Physical: [0x%08X]), size =%d\n",baseAddr, PAL_CPPI4_VIRT_2_PHYS(baseAddr),US_EMBEDDED_DESC_RAM_DESC_SIZE);
    }
    return (0);
}

/**********************************************************************
 *  \brief HAL_DocsisCppiQueueMngInit
 * 
 *  This function initialize the DOCSIS Queue manager.
 *  
 *  
 *  
 *
 *  @return  RCODE_SUCCESS
 *                     
 */
static Int32 HAL_DocsisCppiQueueMngInit(void)
{
    Uint32 regContent = 0;
    Uint32 i;
    volatile Uint32 baseAddr = 0;
    Uint32 usRegionBaseAddr;
    Uint32 usRegionBaseAddrPhys;
    Uint32 regionSize;

    if (debug_enable)
        printk(KERN_INFO "\n*****HAL_DocsisCppiQueueMngInit:\n");

    /* LIAT - NEED TO alloc  memory for Linking RAM 0+1 region */
    /* Initialize the linking RAM region */
    /*HAL_SetMacReg32( DOCSIS_QMGR_LRAM0_BASE_REG, DOCSIS_LINKING_RAM_0_REGION_BASE_ADDR );*/
    /*(*(volatile Uint32*)(HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_LRAM0_BASE_REG))= DOCSIS_LINKING_RAM_0_REGION_BASE_ADDR;*/
    testWriteRegister((HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_LRAM0_BASE_REG),DOCSIS_LINKING_RAM_0_REGION_BASE_ADDR, True);

    /*HAL_SetMacReg32( DOCSIS_QMGR_LRAM0_SIZE_REG, DOCSIS_LINKING_RAM_0_REGION_SIZE );*/
    /*(*(volatile Uint32*)(HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_LRAM0_SIZE_REG))= DOCSIS_LINKING_RAM_0_REGION_SIZE;*/
    testWriteRegister((HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_LRAM0_SIZE_REG),DOCSIS_LINKING_RAM_0_REGION_SIZE, True);
    
    /* Linking RAM #1 not in use but still we need to configure it */
    /*HAL_SetMacReg32( DOCSIS_QMGR_LRAM1_BASE_REG, DOCSIS_LINKING_RAM_1_REGION_BASE_ADDR );*/
    /*(*(volatile Uint32*)(HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_LRAM1_BASE_REG))= DOCSIS_LINKING_RAM_1_REGION_BASE_ADDR;*/
    testWriteRegister((HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_LRAM1_BASE_REG),DOCSIS_LINKING_RAM_1_REGION_BASE_ADDR, True);

    /***********************************************************************/
    /* The base addresses of descriptor regions MUST be in ascending order */
    /***********************************************************************/

    /* Downstream channel Embedded Descriptors configuration */
    /*HAL_SetMacReg32( DOCSIS_QMGR_RAM_BASE_REG(DS_EMBEDDED_DESC_REGION_NUM) , DS_EMBEDDED_DESC_RAM_BASE_ADDR );*/
    /*(*(volatile Uint32*)(HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_RAM_BASE_REG(DS_EMBEDDED_DESC_REGION_NUM)))= DS_EMBEDDED_DESC_RAM_BASE_ADDR;*/
    testWriteRegister((HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_RAM_BASE_REG(DS_EMBEDDED_DESC_REGION_NUM)),DS_EMBEDDED_DESC_RAM_BASE_ADDR, True);

    regContent = 0;
    QMGR_SET_MEM_REGION_DESC_SIZE( regContent, DS_EMBEDDED_DESC_RAM_DESC_ENCODED_SIZE );
    QMGR_SET_MEM_REGION_START_INDEX( regContent, DS_EMBEDDED_DESC_RAM_START_INDEX );
    QMGR_SET_MEM_REGION_NUM_OF_DESC( regContent, DS_EMBEDDED_DESC_RAM_NUM_OF_DESC );
    /*HAL_SetMacReg32( DOCSIS_QMGR_RAM_CTRL_REG(DS_EMBEDDED_DESC_REGION_NUM) , regContent );*/
    /*(*(volatile Uint32*)(HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_RAM_CTRL_REG(DS_EMBEDDED_DESC_REGION_NUM)))= regContent;*/
    testWriteRegister((HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_RAM_CTRL_REG(DS_EMBEDDED_DESC_REGION_NUM)),regContent, True);

    /* Upstream channel Monolithic Descriptors configuration */
    /*HAL_SetMacReg32( DOCSIS_QMGR_RAM_BASE_REG(US_MONOLITHIC_DESC_REGION_NUM) , US_MONOLITHIC_DESC_RAM_BASE_ADDR );*/
    /*(*(volatile Uint32*)(HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_RAM_BASE_REG(US_MONOLITHIC_DESC_REGION_NUM)))= US_MONOLITHIC_DESC_RAM_BASE_ADDR;*/
    testWriteRegister((HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_RAM_BASE_REG(US_MONOLITHIC_DESC_REGION_NUM)),US_MONOLITHIC_DESC_RAM_BASE_ADDR, True);

    regContent = 0;
    QMGR_SET_MEM_REGION_DESC_SIZE( regContent, US_MONOLITHIC_DESC_RAM_DESC_ENCODED_SIZE );
    QMGR_SET_MEM_REGION_START_INDEX( regContent, US_MONOLITHIC_DESC_RAM_START_INDEX );
    QMGR_SET_MEM_REGION_NUM_OF_DESC( regContent, US_MONOLITHIC_DESC_RAM_NUM_OF_DESC );
    /*HAL_SetMacReg32( DOCSIS_QMGR_RAM_CTRL_REG(US_MONOLITHIC_DESC_REGION_NUM) , regContent );*/
    /*(*(volatile Uint32*)(HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_RAM_CTRL_REG(US_MONOLITHIC_DESC_REGION_NUM)))= regContent;*/
    testWriteRegister((HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_RAM_CTRL_REG(US_MONOLITHIC_DESC_REGION_NUM)),regContent, True);

    /* LIAT - NEED TO alloc  memory for US embedded descriptor */
    /* Upstream channel Embedded Descriptors configuration */

    /* ILAN - use "kmalloc()" or PAL_CPPI infrastructure? does infra return physical or virtual?*/ 

    /* We allocate twice the amount of memory reqired - this insures that */
    /* we will have a region with a base address aligned to its size.     */
    regionSize = (US_EMBEDDED_DESC_RAM_DESC_SIZE*US_NUM_OF_EMBEDDE_DESC);
    usRegionBaseAddr = (Uint32)kmalloc((regionSize*2), GFP_KERNEL);
    if (!usRegionBaseAddr) {
        if (debug_enable)
            printk(KERN_INFO "DOCSIS CPPI init Failed to allocate US descriptor region\n");
        return (-1);
    } else
        if (debug_enable)
            printk(KERN_INFO "DOCSIS CPPI init allocated Virtual [0x%08X] (Physical: [0x%08X]) for US descriptor region\n", usRegionBaseAddr, PAL_CPPI4_VIRT_2_PHYS(usRegionBaseAddr));

    /* Aligning the memory pointer to the size of the region */
    usRegionBaseAddr = usRegionBaseAddr - (usRegionBaseAddr%(regionSize)) + regionSize;
    if (debug_enable)
        printk(KERN_INFO "After Alignement Virtual [0x%08X] (Physical: [0x%08X]) (US descriptor region)\n", usRegionBaseAddr, PAL_CPPI4_VIRT_2_PHYS(usRegionBaseAddr));
                          
    usRegionBaseAddrPhys = PAL_CPPI4_VIRT_2_PHYS(usRegionBaseAddr);
    /*HAL_SetMacReg32( DOCSIS_QMGR_RAM_BASE_REG(US_EMBEDDED_DESC_REGION_NUM) , US_EMBEDDED_DESC_DDR_BASE_ADDR );*/
    /*(*(volatile Uint32*)(HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_RAM_BASE_REG(US_EMBEDDED_DESC_REGION_NUM)))= usRegionBaseAddrPhys;*/
    testWriteRegister((HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_RAM_BASE_REG(US_EMBEDDED_DESC_REGION_NUM)),usRegionBaseAddrPhys, True);

    regContent = 0;
    QMGR_SET_MEM_REGION_DESC_SIZE( regContent, US_EMBEDDED_DESC_RAM_DESC_ENCODED_SIZE );
    QMGR_SET_MEM_REGION_START_INDEX( regContent, US_EMBEDDED_DESC_RAM_START_INDEX );
    QMGR_SET_MEM_REGION_NUM_OF_DESC( regContent, US_EMBEDDED_DESC_RAM_NUM_OF_DESC );
    /*HAL_SetMacReg32( DOCSIS_QMGR_RAM_CTRL_REG(US_EMBEDDED_DESC_REGION_NUM) , regContent );*/
    /*(*(volatile Uint32*)(HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_RAM_CTRL_REG(US_EMBEDDED_DESC_REGION_NUM)))= regContent;*/
    testWriteRegister((HAL_MAC_BASE_ADDRESS + DOCSIS_QMGR_RAM_CTRL_REG(US_EMBEDDED_DESC_REGION_NUM)),regContent, True);

    /* Config the DS and US embedded descriptors */
    HAL_DocsisCppiQmngDescInit(usRegionBaseAddr);

    /* Push the embedded DS Descriptors to the free queue */
    if (debug_enable)
        printk(KERN_INFO "Pushing DS descriptors (printing only first BD)\n");
    for (i = 0; i < DS_NUM_OF_EMBEDDE_DESCRIPTORS; i++)   {
        baseAddr = DS_EMBEDDED_DESC_RAM_BASE_ADDR + i*DS_EMBEDDED_DESC_RAM_DESC_SIZE;
        /* Push the descriptor to the free queue */
        /*HAL_SetMacReg32( DS_EMBEDDED_DESC_FREE_QUEUE_REG_OFFSET, baseAddr);        */
        /*(*(volatile Uint32*)(HAL_MAC_BASE_ADDRESS + DS_EMBEDDED_DESC_FREE_QUEUE_REG_OFFSET))= baseAddr;*/
        testWriteRegister((HAL_MAC_BASE_ADDRESS + DS_EMBEDDED_DESC_FREE_QUEUE_REG_OFFSET), baseAddr, (i==0));

    }

    /* Push the embedded US Descriptors to the free queue #60 (0x3C) */
    if (debug_enable)
        printk(KERN_INFO "Pushing US descriptors (printing only first BD)\n");
    for (i = 0; i < US_NUM_OF_EMBEDDE_DESC; i++)   {
        /* TBD Update the Embedded descriptor base address in the DDR */
        /*baseAddr = US_EMBEDDED_DESC_DDR_BASE_ADDR + i*US_EMBEDDED_DESC_RAM_DESC_SIZE;*/
        baseAddr = usRegionBaseAddrPhys + i*US_EMBEDDED_DESC_RAM_DESC_SIZE;
        /* Push the descriptor to the free queue */
        /*HAL_SetMacReg32( US_EMBEDDED_DESC_FREE_QUEUE_REG_OFFSET, baseAddr); */
        /*(*(volatile Uint32*)(HAL_MAC_BASE_ADDRESS + US_EMBEDDED_DESC_FREE_QUEUE_REG_OFFSET))= baseAddr; */
        testWriteRegister((HAL_MAC_BASE_ADDRESS + US_EMBEDDED_DESC_FREE_QUEUE_REG_OFFSET), baseAddr, (i==0));
    }
    return (0);
}

/**********************************************************************
 *  \brief HAL_DocsisCppiDMAChannelInit
 * 
 *  This function initialize the CPPI DMA RX channels for DOCSIS downstream.
 *  
 *  
 *  
 *
 *  @return  RCODE_SUCCESS
 *                     
 */
static Int32
HAL_DocsisCppiDMAChannelInit(void)
{
    /*DMAChnCtrlStatusRegsOvly dmaCtrlRegs = (DMAChnCtrlStatusRegsOvly)DOCSIS_DMA_RX_CH_CTRL_REG_BASE_ADDR;*/
    DMAChnCtrlStatusRegsOvly dmaCtrlRegs = (DMAChnCtrlStatusRegsOvly)(HAL_MAC_BASE_ADDRESS+DOCSIS_DMA_RX_CH_CTRL_REG_BASE_ADDR);
    volatile Uint32 regContent;

    if (debug_enable)
        printk(KERN_INFO "\n*****HAL_DocsisCppiDMAChannelInit:\n");

    if (debug_enable)
        printk(KERN_INFO "Configuring RX Channels:\n");
     regContent = ((Uint32)0x1 << 12) | 0x3b;
     /*HAL_SetMacReg32( DOCSIS_DMA_RX_CH_GLOBAL_CTRL_REG_BASE_ADDR + 0x4, regContent );*/
     /*(*(volatile Uint32*)(HAL_MAC_BASE_ADDRESS + DOCSIS_DMA_RX_CH_GLOBAL_CTRL_REG_BASE_ADDR + 0x4))= regContent;*/
     testWriteRegister((HAL_MAC_BASE_ADDRESS + DOCSIS_DMA_RX_CH_GLOBAL_CTRL_REG_BASE_ADDR + 0x4), regContent, True);

    /* Configure 8 RX channels, one for each DS channel */
    regContent = ((0 << DMA_CHAN_CTRL_RX_EMBEDPKT_FBP0_PNUM_SHIFT) & DMA_CHAN_CTRL_RX_EMBEDPKT_FBP0_PNUM_MASK)      |
                 ((0x1 << DMA_CHAN_CTRL_RX_EMBEDPKT_FBP0_BMGR_SHIFT) & DMA_CHAN_CTRL_RX_EMBEDPKT_FBP0_BMGR_MASK)    |
                 ((0 << DMA_CHAN_CTRL_RX_EMBEDPKT_FBP1_PNUM_SHIFT) & DMA_CHAN_CTRL_RX_EMBEDPKT_FBP1_PNUM_MASK)      |
                 ((0x1 << DMA_CHAN_CTRL_RX_EMBEDPKT_FBP1_BMGR_SHIFT) & DMA_CHAN_CTRL_RX_EMBEDPKT_FBP1_BMGR_MASK)    |
                 ((0 << DMA_CHAN_CTRL_RX_EMBEDPKT_FBP2_PNUM_SHIFT) & DMA_CHAN_CTRL_RX_EMBEDPKT_FBP2_PNUM_MASK)      |
                 ((0x1 << DMA_CHAN_CTRL_RX_EMBEDPKT_FBP2_BMGR_SHIFT) & DMA_CHAN_CTRL_RX_EMBEDPKT_FBP2_BMGR_MASK)    |
                 ((0 << DMA_CHAN_CTRL_RX_EMBEDPKT_FBP3_PNUM_SHIFT) & DMA_CHAN_CTRL_RX_EMBEDPKT_FBP3_PNUM_MASK)      |
                 ((0x1 << DMA_CHAN_CTRL_RX_EMBEDPKT_FBP3_BMGR_SHIFT & DMA_CHAN_CTRL_RX_EMBEDPKT_FBP3_BMGR_MASK));

    /*HAL_SetMacReg32( (Uint32)&dmaCtrlRegs->channelConfig[0].embeddedPktConfigRegA, regContent );*/
/*    (*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[0].embeddedPktConfigRegA)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[0].embeddedPktConfigRegA)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[1].embeddedPktConfigRegA)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[1].embeddedPktConfigRegA)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[2].embeddedPktConfigRegA)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[2].embeddedPktConfigRegA)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[3].embeddedPktConfigRegA)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[3].embeddedPktConfigRegA)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[4].embeddedPktConfigRegA)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[4].embeddedPktConfigRegA)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[5].embeddedPktConfigRegA)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[5].embeddedPktConfigRegA)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[6].embeddedPktConfigRegA)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[6].embeddedPktConfigRegA)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[7].embeddedPktConfigRegA)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[7].embeddedPktConfigRegA)), regContent, True);

    regContent = ((0x3A << DMA_CHAN_CTRL_RX_EMBEDPKT_FDQ_QNUM_SHIFT) & DMA_CHAN_CTRL_RX_EMBEDPKT_FDQ_QNUM_MASK) |
                 ((0x1 << DMA_CHAN_CTRL_RX_EMBEDPKT_FDQ_QMGR_SHIFT) &  DMA_CHAN_CTRL_RX_EMBEDPKT_FDQ_QMGR_MASK) |
                 ((0x3 << DMA_CHAN_CTRL_RX_EMBEDPKT_NUM_SLOT_SHIFT) &  DMA_CHAN_CTRL_RX_EMBEDPKT_NUM_SLOT_MASK) |
                 ((0x0 << DMA_CHAN_CTRL_RX_EMBEDPKT_SOP_SLOT_SHIFT) & DMA_CHAN_CTRL_RX_EMBEDPKT_SOP_SLOT_MASK);

    regContent = 0x0300103a; /* Must be hard-coded because Amos said so!!!*/

    /*HAL_SetMacReg32( (Uint32)&dmaCtrlRegs->channelConfig[0].embeddedPktConfigRegB, regContent );*/
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[0].embeddedPktConfigRegB)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[0].embeddedPktConfigRegB)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[1].embeddedPktConfigRegB)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[1].embeddedPktConfigRegB)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[2].embeddedPktConfigRegB)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[2].embeddedPktConfigRegB)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[3].embeddedPktConfigRegB)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[3].embeddedPktConfigRegB)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[4].embeddedPktConfigRegB)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[4].embeddedPktConfigRegB)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[5].embeddedPktConfigRegB)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[5].embeddedPktConfigRegB)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[6].embeddedPktConfigRegB)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[6].embeddedPktConfigRegB)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[7].embeddedPktConfigRegB)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[7].embeddedPktConfigRegB)), regContent, True);

    /*  If the enable bit is being set, the Rx Channel Global Configuration Register should only be written 
        after ALL of the other Rx Configuration Registers have been initialized. */
    regContent = ((True << DMA_CHAN_CTRL_RX_GLOBAL_CHAN_ENABLE_SHIFT) & DMA_CHAN_CTRL_RX_GLOBAL_CHAN_ENABLE_MASK)   |
                 ((False << DMA_CHAN_CTRL_RX_GLOBAL_CHAN_TD_SHIFT) & DMA_CHAN_CTRL_RX_GLOBAL_CHAN_TD_MASK)          |
                 ((0x0 << DMA_CHAN_CTRL_RX_GLOBAL_SOP_OFFSET_SHIFT) & DMA_CHAN_CTRL_RX_GLOBAL_SOP_OFFSET_MASK)      |
                 ((0x0 << DMA_CHAN_CTRL_RX_GLOBAL_RXCOMP_QNUM_SHIFT) & DMA_CHAN_CTRL_RX_GLOBAL_RXCOMP_QNUM_MASK)    |
                 ((0x1 << DMA_CHAN_CTRL_RX_GLOBAL_RXCOMP_QMGR_SHIFT) & DMA_CHAN_CTRL_RX_GLOBAL_RXCOMP_QMGR_MASK);

    /*HAL_SetMacReg32( (Uint32)&dmaCtrlRegs->channelConfig[0].rxGlobalConfig, regContent );*/
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[0].rxGlobalConfig)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[0].rxGlobalConfig)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[1].rxGlobalConfig)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[1].rxGlobalConfig)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[2].rxGlobalConfig)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[2].rxGlobalConfig)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[3].rxGlobalConfig)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[3].rxGlobalConfig)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[4].rxGlobalConfig)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[4].rxGlobalConfig)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[5].rxGlobalConfig)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[5].rxGlobalConfig)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[6].rxGlobalConfig)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[6].rxGlobalConfig)), regContent, True);
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[7].rxGlobalConfig)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[7].rxGlobalConfig)), regContent, True);

    regContent = DMA_SCHEDULER_ENABLE_BIT | (DOCSIS_CPPI_DMA_NUM_RX_CHANNEL -1);
    /*HAL_SetMacReg32( DOCSIS_DMA_RX_SCHEDULER_CONTROL_REG_ADDR, regContent );*/
    /*(*(volatile Uint32*)(HAL_MAC_BASE_ADDRESS + DOCSIS_DMA_RX_SCHEDULER_CONTROL_REG_ADDR))= regContent;*/
    testWriteRegister((HAL_MAC_BASE_ADDRESS + DOCSIS_DMA_RX_SCHEDULER_CONTROL_REG_ADDR), regContent, True);


    if (debug_enable)
        printk(KERN_INFO "Configuring TX Channel (co-processor):\n");
    /* Configure 1 TX channel for the DS co-processor */
    /*dmaCtrlRegs = (DMAChnCtrlStatusRegsOvly)DOCSIS_DMA_TX_CH_CTRL_REG_BASE_ADDR;*/
    dmaCtrlRegs = (DMAChnCtrlStatusRegsOvly)(HAL_MAC_BASE_ADDRESS+DOCSIS_DMA_TX_CH_CTRL_REG_BASE_ADDR);
    regContent = ((0x1 << DMA_CHAN_CTRL_TX_GLOBAL_CHAN_ENABLE_SHIFT) & DMA_CHAN_CTRL_TX_GLOBAL_CHAN_ENABLE_MASK)     |
                 ((0x0 << DMA_CHAN_CTRL_TX_GLOBAL_CHAN_TD_SHIFT) & DMA_CHAN_CTRL_TX_GLOBAL_CHAN_TD_MASK)             |
                 ((0x1 << DMA_CHAN_CTRL_TX_GLOBAL_DEF_QMGR_SHIFT) & DMA_CHAN_CTRL_TX_GLOBAL_DEF_QMGR_MASK)           |
                 ((0x8 << DMA_CHAN_CTRL_TX_GLOBAL_DEF_QNUM_SHIFT) & DMA_CHAN_CTRL_TX_GLOBAL_DEF_QNUM_MASK);

    /*HAL_SetMacReg32( (Uint32)&dmaCtrlRegs->channelConfig[0].txGlobalConfig, regContent );*/
    /*(*(volatile Uint32*)((Uint32)(&dmaCtrlRegs->channelConfig[0].txGlobalConfig)))= regContent;*/
    testWriteRegister(((Uint32)(&dmaCtrlRegs->channelConfig[0].txGlobalConfig)), regContent, True);

    regContent = DMA_SCHEDULER_ENABLE_BIT | (DOCSIS_CPPI_DMA_NUM_TX_CHANNEL -1); 
    /*regContent = 0;*/  /******* PATCH - block co processor ********/

    /*HAL_SetMacReg32( DOCSIS_DMA_TX_SCHEDULER_CONTROL_REG_ADDR , regContent );*/
    /*(*(volatile Uint32*)(HAL_MAC_BASE_ADDRESS + DOCSIS_DMA_TX_SCHEDULER_CONTROL_REG_ADDR))= regContent;*/
    testWriteRegister((HAL_MAC_BASE_ADDRESS + DOCSIS_DMA_TX_SCHEDULER_CONTROL_REG_ADDR), regContent, True);
    return (0);
}

static Int32
HAL_DocsisCppiInit(void)
{
    if (debug_enable)
        printk(KERN_INFO "Entered HAL_DocsisCppiInit \n");
    HAL_DocsisCppiBufferMngInit();
    if (HAL_DocsisCppiQueueMngInit() != 0) {
        if (debug_enable)
            printk(KERN_INFO "HAL_DocsisCppiInit: HAL_DocsisCppiQueueMngInit Failed!\n");
        return (-1);
    }
    HAL_DocsisCppiDMAChannelInit();
    return (0);
}



/**************************************************************************/
/*! \fn int HAL_DocsisRegisterQosClassifierMacMgmtCB(int (*QosClass_ClassifyMacMng)
 *  (Uint8 *buf, int16 *sfIndex, int16 *phsIndex)) 
 *                                      
 **************************************************************************
 *  \brief register the QOS Classifier Mac Mgmt CB function
 *  \param[in] the QOS Mac Mgmt CB function
 *  \return OK
 **************************************************************************/
int HAL_DocsisRegisterQosClassifierMacMgmtCB(int (*QosClass_ClassifyMacMng)(Uint8 *buf,Int16 *sfIndex,Int16 *phsIndex))
{   
    unsigned long flags;

    local_irq_save(flags);
    HAL_DocsisQosClass_ClassifyMacMngCB = QosClass_ClassifyMacMng;
    local_irq_restore(flags);

    return 0;
}

/***************************************************************************************************************/
/*                                  Internal DDOCSIS CPPI code - end                                           */
/***************************************************************************************************************/


EXPORT_SYMBOL(HAL_DocsisRegisterQosClassifierMacMgmtCB); 


module_init(docsisMngInit);
module_exit(docsisMngCleanup);
MODULE_AUTHOR("Texas Instruments, Inc");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("Cable Modem Management Interface driver");

